# $Id: macros.rb,v 1.8 2004/01/29 16:41:03 brailsmt Exp $

require 'kahakai'
require 'events.rb'
require 'bindings.rb'

$focus = Kahakai::WindowEvFocus.new
$raise = Kahakai::WindowEvRaise.new
$lower = Kahakai::WindowEvLower.new
$menu_focus = kahakai.MenuEvFocus()
$menu_raise = kahakai.MenuEvRaise()
$menu_lower = kahakai.MenuEvLower()
$window_menu_remap_focused = kahakai.WindowEvMenuRemapFocused()
$window_menu_unmap = kahakai.WindowEvMenuUnmap()
$screen_menu_remap_focused = kahakai.ScreenEvMenuRemapFocused()
$screen_menu_unmap = kahakai.ScreenEvMenuUnmap()
$move = kahakai.WindowEvMove()
$moveOpaque = Kahakai::WindowEvMoveOpaque.new
$smartResize = Kahakai::WindowEvSmartResize.new
$smart_resize_opaque = kahakai.WindowEvSmartResizeOpaque()
$endMoveResize = Kahakai::WindowEvEndMoveResize.new
$menu_end_move_resize = _processFunction("EndMoveResize", "menu")

$next_item = kahakai.MenuEvNextItem()
$unmap_tree = kahakai.MenuEvUnmapTree()
$unmap_submenus = kahakai.MenuEvUnmapSubmenus()
$remap_submenu_focused = kahakai.MenuEvRemapSubmenuFocused()
$exec = kahakai.MenuEvExec()
$func = kahakai.MenuEvFunc()

$screenEndMoveResize = Kahakai::ScreenEvEndMoveResize.new
$screenMenuRemap = Kahakai::ScreenEvMenuRemapFocused.new
$screenMenuUnmap = Kahakai::ScreenEvMenuUnmap.new

$windowMenuRemap = Kahakai::WindowEvMenuRemapFocused.new
$windowMenuUnmap = Kahakai::WindowEvMenuUnmap.new

def clickFocus passthrough=true
    currentScreen.bind 'window.all', 
        ButtonPress.new('Button1'), $focus, replay=passthrough
end

def sloppyFocus
    currentScreen.bind 'window.all', EnterNotify.new(), $focus
end

def mouseFocus
    sloppyFocus
    currentScreen.bind EnterNotify.new, $focus
end

def focusNewWindows
    currentScreen.bind 'window.passiveclient', MapRequest.new, $focus
end

def wheelStacking raiseButton=5, lowerButton=4, modifier=Alt
    raiseButton = "Button#{raiseButton}"
    lowerButton = "Button#{lowerButton}"

    ['window.title', 'window.label'].each do |target| 
        currentScreen.bind target, ButtonPress.new(lowerButton), $lower 
        currentScreen.bind target, ButtonPress.new(raiseButton), $raise
    end 
    currentScreen.bind 'window.all', 
        ButtonPress.new(modifier, lowerButton), $lower
    currentScreen.bind 'window.all', 
        ButtonPress.new(modifier, raiseButton), $raise
end

def clickToRaise passthrough=true
    currentScreen.bind 'window.all', 
        ButtonPress.new('Button1'), $raise, replay=passthrough    
end

def mouseRaise 
    currentScreen.bind 'window.all', EnterNotify.new, $raise
end

def windowSnapping pixels=15
    currentScreen.config.snapping     = true
    currentScreen.config.snapDistance = pixels
end

EDGE_ACTIONS = {
    'edge.n' => Kahakai::ScreenEvMoveViewportUp.new,
    'edge.s' => Kahakai::ScreenEvMoveViewportDown.new,
    'edge.e' => Kahakai::ScreenEvMoveViewportRight.new,
    'edge.w' => Kahakai::ScreenEvMoveViewportLeft.new
}

#  Installs MoveViewport{Up,Down,Left,Right} actions for edges,
#  activated by the specified modifier(s), or by no modifiers if
#  modifiers is 0.  This also allows you to drag windows around if
#  you're holding your appropriate modifier(s) while doing so.
def viewportSurf modifier=-Ctrl+Alt
    EDGE_ACTIONS.each_pair do |edge, action|
        currentScreen.bind edge, EnterNotify.new(modifier), action
    end
end

# Allows you to "knock" on screen edges with a mouse button to flip
# viewports.
def viewportKnock button=1, modifier=-Ctrl+Alt
    button = 'Button#{button}'
    EDGE_ACTIONS.each_pair do |edge, action|
        currentScreen.bind edge, ButtonPress.new(modifier, button), action
    end
end

#  Keys for switching desktops.  Parameter "modifier" should be the
#  modifiers you wish to use, keys should either be a string
#  containing a %d so that the desktop number will be filled in for
#  each (for e.g. F1,F2,etc), or a string containing a list of keys
#  seperated by commas that will switch to each respective viewport.
#
#  This string-list of keys should have the same or more keys than
#  the number of desktops.
def desktopKeys modifier=Ctrl+Alt-AltGr, keys="%d"
    num_desktops = currentScreen.desktop_list.length

    if keys.include? '%d'
        ourKeys = (1..num_desktops).collect {|i| keys.sub("%d", i.to_s)}
    elsif keys.include? ','
        ourKeys = keys.split ','

        if ourKeys.length < num_desktops
            warn "desktopKeys(): only #{ourKeys.length} defined, #{num_desktops} needed"              
        end
    else
        warn "Wrong format for keys argument"
        return
    end

    goToDesktop = Kahakai::ScreenEvGoToDesktop.new

    1.upto(num_desktops) do |i|
        currentScreen.bind 'all', KeyPress::new(modifier, ourKeys[i-1]), 
            goToDesktop, i.to_s
    end
end

# Installs an action for Alt+Tab.  Default is TaskSwitcher, you can
# change this by passing an argument of a different action.
def altTab action="TaskSwitcher"
    currentScreen.bind 'all', KeyPress.new(-Ctrl-AltGr+Alt, 'Tab'), action
end

def rootMenuClick buttons
    buttons.each do |button|
        currentScreen.bind ButtonPress::new(-Ctrl-Alt, "Button#{button}"), 
            $screenMenuRemap, 'rootmenu'
    end
end

def rootMenuUnmap buttons
    buttons.each do |button|
        currentScreen.bind ButtonPress::new(-Ctrl-Alt, "Button#{button}"), 
            $screenMenuUnmap, 'rootmenu'
    end
end

DECOR_TARGETS = ['window.label', 'window.title', 'window.handle', 
                 'window.leftgrip', 'window.rightgrip']

def windowMenuClick buttons
    buttons.each do |button|
        DECOR_TARGETS.each do |target|
            currentScreen.bind target, 
                               ButtonPress.new(-Ctrl-Alt, "Button#{button}"), 
                               $windowMenuRemap, 'window'
        end
    end
end

def windowMenuUnmap buttons
    buttons.each do |button|
        DECOR_TARGETS.each do |target|
            currentScreen.bind target, 
                               ButtonPress.new(-Ctrl-Alt, "Button#{button}"), 
                               $windowMenuUnmap, 'window'
        end
    end
end

def desktopWheelSCroll upaction='PreviousDesktop', downaction='NextDesktop', modifier=nil
    if modifier 
        up = [modifier, 'Button4']
        down = [modifier, 'Button5']
    else
        up   = ['Button4']
        down = ['Button5']
    end
    
    currentScreen.bind ButtonPress.new(*up),   upaction
    currentScreen.bind ButtonPress.new(*down), downaction
end

def smartWindowPlacement
    currentScreen.bind 'window.all', MapRequest.new(), 'MoveWindowToSmartPlace'
end

def moveWindowWithModifier modifier=Alt
    currentScreen.bind 'window.all', ButtonPress.new(modifier, 'Button1'), 
                       $moveOpaque
    currentScreen.bind 'all', 
                       ButtonRelease.new(Kahakai::MoveResizeMask, 'Button1'), 
                       $screenEndMoveResize, replay=true
end

# FIXME: Doesn't work right yet
def resizeWindowWithModifier modifier=Alt
    currentScreen.bind 'window.all', ButtonPress.new(modifier, 'Button3'), 
                       $smartResize
    currentScreen.bind ButtonRelease.new(Kahakai::MoveResizeMask, 'Button3'), 
                       $screenEndMoveResize
    currentScreen.bind 'window.all', 
                       ButtonRelease.new(Kahakai::MoveResizeMask, 'Button3'), 
                       $endMoveResize
end

def lowerWindowWithModifier modifier=Alt
    currentScreen.bind 'window.all', ButtonPress.new(modifier, 'Button2'),
                       $lower
end

def moveByTitlebar buttons
    buttons.each do |button|
        ['window.label', 'window.title', 'window.handle'].each do |target|
            currentScreen.bind target, 
                               ButtonPress.new("Button#{button}"), $moveOpaque
            currentScreen.bind target, 
                               ButtonRelease.new(Kahakai::MoveResizeMask, 
                                                 "Button#{button}"), 
                               $endMoveResize
        end
    end
end

def doubleClickTitlebarShade buttons
    buttons.each do |button|
        ['window.label', 'window.title'].each do |target|
            currentScreen.bind target, 
                               DoubleClick.new(-Ctrl-Alt, "Button#{button}"),
                              'ToggleShade' 
        end
    end
end

def resizeWithGrips buttons
    buttons.each do |button|
        currentScreen.bind 'window.leftgrip', 
                           ButtonPress.new("Button#{button}"), 'ResizeLeft'
        currentScreen.bind 'window.rightgrip', 
                           ButtonPress.new("Button#{button}"), 'ResizeRight'   
    end
end

def buttonAction buttonName, action, button='Button1'
    currentScreen.bind "window.button.#{buttonName}", 
                       ButtonRelease.new(-Ctrl-Alt, button), action
end

def shadeButton
    buttonAction 'shade', 'ToggleShade'
end

def closeButton
    buttonAction 'close', 'Close'
end

def maximizeButton
    buttonAction 'maximize', 'ToggleMaximize'
    buttonAction 'maximize', 'MaximizeVertically', 'Button2'    
    buttonAction 'maximize', 'MaximizeHorizontally', 'Button3'
end

def windowButtons
    shadeButton
    closeButton
    maximizeButton
end

def mouseMenuNavigation
    screen = currentScreen

    # Menu itself
    screen.bindList 'menu.all',
                    [[ButtonPress.new(-Ctrl+Alt, 'Button1'), 'Raise'],
                     [ButtonPress.new(-Ctrl+Alt, 'Button1'), 'MoveOpaque'],
                     [ButtonPress.new(-Ctrl+Alt, 'Button1'), 'UnLinkMenu'],
                     [ButtonRelease.new(Kahakai::MoveResizeMask, 'Button1'),
                      'EndMoveResize'],
                     [ButtonPress.new(-Ctrl-Alt, 'Button3'), 'UnmapTree']]
    # Items
    ['menu.item', 'menu.sub', 'menu.checkbox'].each do |target|
        screen.bindList target, 
                        [[ButtonPress.new(-Ctrl-Alt, 'Button1'), 'Exec'],
                         [ButtonPress.new(-Ctrl-Alt, 'Button1'), 'Func'],
                         [ButtonPress.new(-Ctrl-Alt, 'Button1'), 'UnmapTree']]
    end

    # Title
    screen.bindList 'menu.title',
                    [[ButtonPress.new('Button1'), 'Raise'],
                     [ButtonPress.new(-Alt, 'Button1'), 'UnLinkMenu'],
                     [ButtonPress.new(-Alt, 'Button1'), 'Move'],
                     [ButtonPress.new('Button3'), 'UnmapSubmenus'],
                     [ButtonPress.new('Button3'), 'UnmapMenu']]
    # Item
    screen.bindList 'menu.item', 
                    [[EnterNotify.new(), 'UnmapSubmenus'],
                     [ButtonRelease.new('AnyButton'), 'UnmapTree']]
    # Checkbox
    screen.bind 'menu.checkbox', EnterNotify.new(), 'UnmapSubmenus'
    
    # Submenu
    screen.bind 'menu.sub', EnterNotify.new(), 'MapSubmenuFocusedOnly'
    screen.bind 'menu.sub', ButtonPress.new('AnyButton'), 'RemapSubmenuFocused'
end

def keyboardMenuNavigation
    screen = currentScreen

    ['menu.item', 'menu.sub', 'menu.checkbox'].each do |target|
        screen.bindList target, 
                        [[KeyPress.new('Tab') , 'NextItem'],
                         [KeyPress.new('Down'), 'NextItem'],
                         [KeyPress.new('Up'),   'PreviousItem'],
                         [KeyPress.new('Left'), 'UnmapSubmenus'],
                         [KeyPress.new('Escape'), 'UnmapTree'],
                         [KeyPress.new('Return'), 'Exec'],
                         [KeyPress.new('Return'), 'Func'],
                         [KeyPress.new('Return'), 'UnmapTree']]
    end
    
    screen.bindList 'menu.item', 
                    [[KeyRelease.new('Alt_L'), 'UnmapTree'],
                     [KeyRelease.new('Alt_L'), 'Func'],
                     [KeyPress.new('Return'), 'UnmapTree']]
    
    screen.bind 'menu.checkbox', KeyPress.new('Space'), 'Exec'
    screen.bind 'menu.checkbox', KeyPress.new('Space'), 'Func'

    screen.bind 'menu.sub', KeyPress.new('Right'), 'RemapSubmenuFocused'
    screen.bind 'menu.sub', KeyPress.new('Return'), 'RemapSubmenuFocused'
end
