/* $Id: ChildWindow.hh,v 1.7 2003/11/29 04:47:29 pervago Exp $ */

/**
 * @file   ChildWindow.hh
 * @date   31 Aug 2003
 *
 * @brief Defines KaChildWindow class which used for grips, handle
 * bar, buttons, etc.
 *
 */
#ifndef __ChildWindow_hh
#define __ChildWindow_hh

#include "Types.hh"
#include "Kahakai.hh"
#include "WindowAttributes.hh"
#include "Style.hh"

#include "DrawOp.hh"

class ImageControl;
class Texture;

class KaChildWindow : public WindowObject {
public:
    KaChildWindow(KaWindow *, Window, int);
    virtual ~KaChildWindow();

    virtual void render();    
    void update();
    void clearCache();

    Display *display;
    KaWindow *ka;
    KaScreen *kascreen;
    WindowAttributes attrib;
    TexturePtr focusedTexture, unfocusedTexture;
    int g_x, g_x2;

    inline void updateAttributes() {
        XMoveResizeWindow(display, id,
                          attrib.getX(), attrib.getY(),
                          attrib.getWidth(), attrib.getHeight());
    }

    XftDraw *xftdraw;
protected:
    Pixmap pixmap;
    ImageControl *control;
    
    TexturePtr focusedRenderedTexture, unfocusedRenderedTexture;

    bool isVisible();

    void createWindow(Window parent, 
                      int create_mask=0, XSetWindowAttributes* attr_set=NULL);
    void renderTexture();
};

class KaWindowFrame: public KaChildWindow {
public:
    KaWindowFrame(KaWindow* kawin, Window parent);
    //    virtual void Render();
};

class KaWindowLabel: public KaChildWindow {
public:
    KaWindowLabel(KaWindow* kawin, Window parent);
    ~KaWindowLabel();
    //    virtual void Render() {};
};

class KaWindowHandle: public KaChildWindow {
public:
    KaWindowHandle(KaWindow* kawin, Window parent);
    ~KaWindowHandle();
    virtual void render();
private:
    TexturePtr draw(bool focused);
};

class KaWindowButton: public KaChildWindow {
public:
    KaWindowButton(KaWindow* kawin, Window parent);
    //    virtual void Render();

    ButtonStyle *bstyle;
    bool pressed;
    
    ButtonStateStyle * getButtonStateStyle();
    bool isActive();
};

typedef enum {
    Right,
    Left
} Side;

class KaWindowGrip: public KaChildWindow {
public:
    KaWindowGrip(KaWindow* kawin, Window parent, 
                 Side side);
    ~KaWindowGrip();
    virtual void render();
private:
    TexturePtr draw(bool focused);
};

class KaWindowEdge: public KaChildWindow {
public:
    KaWindowEdge(KaWindow* kawin, Window parent, 
                 Side side);
    ~KaWindowEdge();
    virtual void render();
private:
    TexturePtr draw(bool focused);
};

#endif
