/* $Id: DesktopMenu.cc,v 1.2 2004/02/07 22:43:44 slide_o_mix Exp $ */

/**
 * @file   DesktopMenu.cc
 * @date   Sun Jan  4 19:39:51 2004
 * 
 * @brief Menu listing all desktops except the current one and
 * allowing either going to a desktop or joining/parting/sending to of
 * current window.
 * 
 * 
 */

#include "DesktopMenu.hh"
#include "MenuItem.hh"

#include <vector>

using namespace std;

/**
 * @fn    DesktopMenu(WindowFunction* function, char *n, bool isPart) :
 * @brief Constructor for DesktopMenu class
 *
 * Creates a DesktopMenu object, used for switching between desktops.
 * This is a KaMenu with some extra functionality. 
 *
 * @param isPart Specifies whether this function is a 'Part desktop X'
 * function which means we should exclude the desktops on which the
 * window is not present, as opposed to all the rest which include
 * desktops where the window already exists.
 *
 */
DesktopMenu::DesktopMenu(WindowFunction* func, char *n, bool _isPart) :
    KaMenu(n), function(func), isPart(_isPart) {
    KaMenuItem *m;

    m = new KaMenuItem("");
    m->type = MenuTitleType;
    AddItem(m);

    ext_type = DesktopExtMenuType;
}

/**
 * @fn    Build(KaScreen *kascreen)
 * @brief Builds DesktopMenu
 *
 * Overloaded Build function to make DesktopMenu build a menu from
 * current dsktops.
 *
 * @param kascreen KaScreen to map menu on
 */
void DesktopMenu::Build(KaScreen *kascreen, Window win_id) {
    std::vector<Desktop *> &desktop_list = kascreen->desktop_list;
    vector<Desktop *>::iterator it;

    bool onWindow = win_id != 0;

    map<Window, WindowObject *>::iterator hit;
    unsigned int desktop_mask;
    if (onWindow) {
        hit = kascreen->kahakai->window_table.find(win_id);
        if (hit == kascreen->kahakai->window_table.end())
            return;
        desktop_mask = ((KaWindow *) (*hit).second)->desktop_mask;
    }

    LISTCLEAR(item_list);

    for (it = desktop_list.begin(); it != desktop_list.end(); it++) {
        Desktop &d = **it;

        if (onWindow) {
            // If command is 'part desktop X', then we skip the desktops
            // we're not on. Else, skip the desktops we're on.
            if (isPart && (~desktop_mask) & (1L << d.number)) 
                continue;
            else if (desktop_mask & (1L << d.number))
                continue;
        }

        char str[20];

        //sprintf(str, "Desktop %d", d.number + 1);
	
        KaMenuItem *m = new KaMenuItem(strdup(d.name.c_str()));

        m->type = MenuItemType;
        sprintf(str, "%d", d.number);
        m->param = strdup(str);

        if (onWindow) {
            m->func_mask |= MenuWFuncMask;
            m->func_mask1 |= MenuWFuncMask;
            m->wfunc = function;
            m->wf = win_id;
        } else {
            m->func_mask |= MenuRFuncMask;
            m->func_mask1 |= MenuRFuncMask;
            m->rfunc = (ScreenFunction*)function;
            m->wf = 0;
        }
        AddItem(m);
    }
    KaMenu::Build(kascreen);
}
