/* $Id: Event.hh,v 1.16 2004/02/04 21:53:28 brailsmt Exp $ */
 
/**
 * @file   Event.hh
 * @date   11-May-2001 11:48:03
 *
#define EVENT.HH
 * @brief Definition of EventHandler class  
 *
 * Function declarations and variable definitions for EventHandler class.
 *
 */

#ifndef __Event_hh
# define __Event_hh

#include <sys/time.h>

#include "Types.hh"

extern "C" {
#include <X11/Xlib.h>
}
#include <set>

struct EventDetail {
    unsigned int type, mod, detail;
};

#define MoveResizeMask (1L << 25)

#define DoubleClick 36

class EventHandler {
public:
    EventHandler(Kahakai *);
    virtual ~EventHandler(void);

    void EventLoop(std::set<int> *, XEvent *);
    void HandleEvent(XEvent *);
    void CheckDoubleClick(XEvent *, EventDetail *);
    void PossiblyDemoteDoubleClick(KaWindow *, EventDetail *);
    void EvExpose(XExposeEvent *);
    void EvFocus(XFocusChangeEvent *);
    void EvUnmapDestroy(XEvent *);
    void EvConfigureRequest(XConfigureRequestEvent *);
    void EvAct(XEvent *, Window, EventDetail *);

    inline bool IsButtonEventType(unsigned int type) {
        return type==ButtonPress || type==ButtonRelease || type==DoubleClick;
    }
    inline bool IsKeyEventType(unsigned int type) {
        return type == KeyPress || type == KeyRelease;
    }

    XEvent *event;
    std::set<int> *empty_return_mask;
    std::set<int> *moveresize_return_mask;
    std::set<int> *menu_viewport_move_return_mask;

    // this is the current state of things as far as moving/resizing things
    // are concerned.  normally it's EndMoveResizeType; if a window is being
    // moved, then it's MoveType or MoveOpaqueType, and if a window is being
    // resized then it's either ResizeType or ResizeOpaqueType.  this seems
    // like an awfully weird way to track state, could probably stand to be
    // made more sane..
    int move_resize;

    Window focused;

private:
    void EvProperty(XPropertyEvent *);
    void EvColormap(XColormapEvent *);
    void EvMapRequest(XMapRequestEvent *);
    void EvClientMessage(XEvent *, EventDetail *);
 
    Kahakai *kahakai;
    ResourceHandler *rh;
    Window last_click_win;
    unsigned int last_button;
    struct timeval last_click;
};

Bool eventmatch(Display *, KaAction *, EventDetail *, unsigned int = 0);

#endif // __EventHandler_hh
