/* $Id: ExpressionEvaluator.hh,v 1.4 2003/11/29 02:42:14 pervago Exp $ */

/**
 * @file   ExpressionEvaluator.hh
 * @date   20-Aug-2003
 *
 * @brief  Class for evaluation of expressions used in the new style format.
           (e.g. baseFontSize*1.1 + 3)
 *
 */

#ifndef __ExpressionEvaluator_hh
# define __ExpressionEvaluator_hh

# include "Types.hh"

# include <string>
# include <map>
# include <stack>

struct parse_exception: public std::exception {
    parse_exception(const std::string& expression_, 
                    const char* stop_) throw(): expression(expression_), 
                                                stop(stop_) {};
    ~parse_exception() throw() {};

    const char* what() const throw() {
        return ("Parse error in '" + expression + 
                "', stopped at " + stop).c_str(); 
    }

    const std::string expression;
    const char* stop;
};

class ExpressionEvaluator
{
public:
    ExpressionEvaluator(const std::map<std::string, float>& variables);
    ~ExpressionEvaluator();

    float evaluate(const std::string& expression);
private:
    const std::map<std::string, float>& variables;
};

#endif
