/* $Id: Functions.hh,v 1.11 2004/02/02 21:47:25 brailsmt Exp $ */

/**
 * @file   Functions.hh
 * @date   06-Jun-2003
 *
 * @brief Definition of *Function family of function objects (functors)
 *
 */

#ifndef __Functions_hh
# define __Functions_hh

# include <Types.hh>

extern "C" {
# include <X11/Xlib.h>
}

class Function
{
public:
    bool requiresParameter;
};

class WindowFunction: public Function
{
public:
    virtual void operator()(KaWindow*, XEvent*, KaAction*)=0;
};


class MenuFunction: public Function
{
public:
    virtual void operator()(KaMenuItem*, XEvent*, KaAction*)=0;
};

// ScreenFunction inherits from MenuFunction & WindowFunction
// because KaScreen objects is obtainable from KaWindow and KaMenuItem
class ScreenFunction: public MenuFunction, public WindowFunction
{
public:
    ScreenFunction() {};
	void operator()(KaMenuItem* item, XEvent* event, KaAction* action);
	void operator()(KaWindow* window, XEvent* event, KaAction* action);
    virtual void operator()(KaScreen*, XEvent*, KaAction*)=0;
};

class SimpleFunction: public ScreenFunction
{
public:
    inline void operator()(KaScreen* screen, XEvent* event, 
                           KaAction* action) {
        (*this)(event, action);
    };
    virtual void operator()(XEvent*, KaAction*)=0;
};

#endif
