/* $Id: Image.hh,v 1.14 2003/11/20 22:34:30 pervago Exp $ */

/**
 * @file   Image.hh
 * @date   18-Jul-2001 01:45:32
 *
 * @brief Definition of KaImageControl classes
 *
 * Function declarations and variable definitions for 
 * KaImageControl class.
 *
 * Copyright (c) 1997 - 2000 Brad Hughes (bhughes@tcac.net)
 *
 */

#ifndef   __Image_hh
# define   __Image_hh

# include "Types.hh"

extern "C" {
# include <X11/extensions/Xrender.h>
# include <X11/Xft/Xft.h>
# ifdef    PIXMAP
#  include <Imlib2.h>
# endif // PIXMAP
}
#include <list>

class KaColor {
private:
    int allocated;
    unsigned char red, green, blue, transparency;
    unsigned long pixel;

    XRenderColor xrenderc;

    XftColor xftc;

public:
    KaColor(char r = 0, char g = 0, char b = 0)
        { red = r; green = g; blue = b; pixel = 0; allocated = 0; }

    inline const int &isAllocated(void) const { return allocated; }

    inline const unsigned char &getRed(void) const { return red; }
    inline const unsigned char &getGreen(void) const { return green; }
    inline const unsigned char &getBlue(void) const { return blue; }
    inline const unsigned long &getPixel(void) const { return pixel; }

    inline void setAllocated(int a) { allocated = a; }
    inline void setPixel(unsigned long p) { pixel = p; }
    void setRGB(unsigned short, unsigned short, unsigned short);

    inline XRenderColor *getXRenderColor(void) { return &xrenderc; }

    void setXftTransparency(unsigned char);
    inline XftColor *getXftColor() { return &xftc; }
};

template <typename Z> inline Z kamin(Z a, Z b) { return ((a < b) ? a : b); }
template <typename Z> inline Z kamax(Z a, Z b) { return ((a > b) ? a : b); }


class KaImageControl {
private:
    bool dither;
    Display *display;
    KaScreen *kascreen;
    Visual *visual;
    Colormap colormap;

    XColor *colors;
    Window window;
    int colors_per_channel, ncolors, screen_number, screen_depth,
        bits_per_pixel, red_offset, green_offset, blue_offset,
        red_bits, green_bits, blue_bits;
    unsigned char red_color_table[256], green_color_table[256],
        blue_color_table[256];
    unsigned int *grad_xbuffer, *grad_ybuffer, grad_buffer_width,
        grad_buffer_height;
    unsigned long *sqrt_table, cache_max;

    typedef struct Cache {
        Pixmap pixmap;

        unsigned int count, width, height;
        unsigned long pixel1, pixel2, texture;
    } Cache;

    std::list<Cache *> *cache;

protected:
    Pixmap searchCache(unsigned int, unsigned int, unsigned long, KaColor *,
                       KaColor *);

public:
    KaImageControl(Display *, KaScreen *, bool = false, int = 4,
                   unsigned long = 200l);
    virtual ~KaImageControl(void);

    inline Display *getDisplay(void) { return display; }
    inline int getScreen(void) { return screen_number; }
    inline KaScreen *getKaScreen(void) { return kascreen; }
    inline Visual *getVisual(void) { return visual; }
    inline Colormap getColormap(void) { return colormap; }
    inline const Window &getDrawable(void) const { return window; }
    inline const int &getBitsPerPixel(void) const { return bits_per_pixel; }
    inline const int &getDepth(void) const { return screen_depth; }
    inline const int &getColorsPerChannel(void) const
        { return colors_per_channel; }
    void installRootColormap(void);
    void setDither(bool d) { dither = d; }
    void setColorsPerChannel(int);
    void parseColor(KaColor *, const char * = 0);
    void setXRootPMapId(bool);
private:
    unsigned long getColor(const char *, unsigned short *, unsigned short *,
                           unsigned short *);
};

#endif // __Image_hh
