/**
 * @file   ImageControl.hh
 * @date   Sat Oct  4 23:14:43 2003
 * 
 * @brief Declaration of ImageControl class which handles rendering
 *        drawing operations into pixmap.
 * 
 * 
 */

#ifndef __ImageControl_hh
#define __ImageControl_hh

extern "C" {
#  include <X11/X.h>
#  include <X11/Xatom.h>
#  include <X11/Xmd.h>
}

#include "DrawOp.hh"

#include <boost/weak_ptr.hpp>

using boost::weak_ptr;

/* The ImageControl class.. Takes care of rendering a vector of DrawOps... */
class ImageControl {
public:
    ImageControl(KaScreen* screen, Display *, int, Visual *, Colormap);
    ~ImageControl();

    Pixmap render(Drawable, const DrawOps&, int, int);
    Pixmap render(Drawable, const DrawOps&, int, int, int, int);
    Pixmap render(Drawable, shared_ptr<Texture>, int, int, int, int);

    shared_ptr<ImImage> renderToImage(shared_ptr<Texture>, int, int, int, int);
    Pixmap renderImage(Drawable parent, shared_ptr<ImImage> image);
    void   renderImageOnDrawable(Drawable drawable, shared_ptr<ImImage> image,
                                 int x, int y);

    shared_ptr<ImImage> getTextureImage(shared_ptr<Texture> texture,
                                        int width, int height);

    void clearCache();
    void cleanup();

    void render_on_drawable(Drawable d,
                            shared_ptr<Texture> texture, 
                            int x, int y,
                            unsigned int width,
                            unsigned int height);

    inline void free_pixmap(Pixmap pixmap) { 
        imlib_free_pixmap_and_mask(pixmap); 
        //        if(rootImage) delete rootImage;
    };

private:
    typedef std::map<weak_ptr<Texture>, std::map<std::pair<int, int>, 
                                                 shared_ptr<ImImage> > > 
    Cache;

    typedef std::map<weak_ptr<Texture>, std::map<std::pair<int, int>, int> > 
    CacheUseCount;

    
    void getRootImage();
    void timeout();

    Cache cache;
    CacheUseCount cacheCount;

    ImContext *ctx;
    Atom root_atom;
    Pixmap rootPixmap;
    Display* display;
    KaScreen* kascreen;
    //    ImImage *rootImage;
    int screen;
};


#endif // __ImageControl_hh
