/**
 * @file   Imlib2pp.hh
 * @date   1-Octl-2003 15:40:00
 *
 * @brief Definition of Imlib2 C++ wrapper classes
 *
 * Function definition of the Imlib2 C++ wrapper classes
 *
 */

#ifndef __Imlib2pp_hh
# define __Imlib2pp_hh

extern "C" {
# include <X11/Xlib.h>
# include <Imlib2.h>
}

#include <string>
#include <vector>
#include <utility>

class ImColor {
public:
    ImColor(): red(0), green(0), blue(0), alpha(0) {};
    ImColor(int r, int g, int b, int alpha = 255);

    void set() const;

    ImColor get_hilite() const;
    ImColor get_lolite() const;

    char red, green, blue, alpha;
};


class ImPolygon {
public:
    ImPolygon();
    ~ImPolygon();

    void add_point(int x, int y);
    void get_bounds(int& left, int& top, int& right, int& bottom);
    bool contains(int x, int y);

    ImlibPolygon polygon;
};

class ImColorRange {
public:
    ImColorRange(const std::vector<std::pair<ImColor, int> >& _range);
    ~ImColorRange();

    void set() const;
private:
    Imlib_Color_Range colorRange;
};


class ImImage {
public:
    ImImage();
    ImImage(int, int);
    ImImage(const std::string&);
    ImImage(int, int, DATA32 *);
    ImImage(Drawable, Pixmap, int, int, int, int, char);
    ImImage(Drawable, Pixmap, int, int, int, int, int, int, char, char);
    ~ImImage();

    bool valid() const;
    
    void set() const;
    void set_has_alpha(bool has_alpha);

    void clear();
    void clear(const ImColor& color);

    ImImage crop(int x, int y, int width, int height);
    
    int get_width() const;
    int get_height() const;

    void set_border(int, int, int, int);
    void set_border(int);
    void get_border(int&, int&, int&, int&);

    void blend_part_scaled(const ImImage&, bool, int, int, int, int, int, int, int, int);
    void blend_scaled(const ImImage&, bool, int, int, int, int);
    void blend(const ImImage&, bool, int, int);

    void draw_line(ImColor, int, int, int, int);
    void draw_rectangle(ImColor, int, int, int, int);
    void fill_rectangle(ImColor, int, int, int, int);
    void fill_color_range_rectangle(const ImColorRange&, int, int, int, int,
                                    double angle=0);

    void draw_polygon(ImPolygon, ImColor, bool closed = true);
    void fill_polygon(ImPolygon, ImColor);

    void draw_bevel(ImColor, unsigned int, int);

    void set_format(const std::string&);
    void save(const std::string&);

    void apply_fixed_alpha(unsigned char alpha);

    void render_pixmaps(Drawable, Pixmap *, Pixmap *) const;
    void render_pixmaps(Drawable, Pixmap *, Pixmap *, int, int) const;
    void free_pixmap_and_mask(Pixmap);
    void render_on_drawable(Drawable, int, int) const;
    void render_on_drawable(Drawable, int, int, int, int);
    void render_on_drawable(Drawable, int, int, int, int, int, int, int, int);    
    
private:
    ImImage(const ImImage& img) {};

    Imlib_Image image;    
};


class ImContext {
public:
    ImContext(Display *, Visual *, Colormap);
    ~ImContext();
    void push();
    void pop();
    
private:
    Imlib_Context context;
};

#endif
