/* $Id: Kahakai.hh,v 1.28 2004/02/04 21:53:28 brailsmt Exp $ */

/**
 * @file   Kahakai.hh
 * @date   02-May-2001 00:49:45
 *
 * @brief Definition of Kahakai and WindowObject classes
 *
 * Function declarations and variable definitions for Kahakai class.
 *
 */

#ifndef __Kahakai_hh
# define __Kahakai_hh

# include "Types.hh"
//#include "ka_logger.hh"

extern "C" {
# include <X11/Xlib.h>
# ifdef XINERAMA
#  include <X11/extensions/Xinerama.h>
# endif
}
# include <vector>
# include <string>
# include <map>
# include <sstream>

# define __m_wastrdup(_str) (((__m_wastrdup_tmp =                       \
                               new char[strlen(_str) + 1]) &&           \
                              sprintf(__m_wastrdup_tmp, "%s", _str))?   \
                             __m_wastrdup_tmp : __m_wastrdup_tmp)

class WindowObject {
public:
    inline WindowObject(Window win_id, int win_type) {
    id = win_id;
    type = win_type;
    actionlist = NULL;
}

    Window id;
    int type;
    std::vector<KaAction *> *actionlist;
};

#define EastType   1
#define WestType  -1
#define SouthType  1
#define NorthType -1

struct kaoptions {
    char *display;
    char *rcfile;
    char *stylefile;
    char *actionfile;
    char *menufile;
    char *scriptconfig;
};

#define LISTDEL(list)                           \
    while (! list.empty()) {                    \
        delete list.back();                     \
        list.pop_back();                        \
    }

#define LISTDELITEMS(list)                      \
    while (! list.empty()) {                    \
        delete list.back();                     \
    }

#define LISTPTRDEL(list)                        \
    while (! list->empty()) {                   \
        delete list->back();                    \
        list->pop_back();                       \
    }

#define LISTCLEAR(list)                         \
    while (! list.empty()) {                    \
        list.pop_back();                        \
    }

#define LISTPTRCLEAR(list)                      \
    while (! list->empty()) {                   \
        list->pop_back();                       \
    }                                           \
        delete list;

#define LISTPTRDELITEMS(list)                   \
    while (! list->empty()) {                   \
        delete list->back();                    \
    }

#define MAPCLEAR(map)                           \
    while (! map.empty()) {                     \
        map.erase(map.begin());                 \
    }

#define MAPPTRCLEAR(map)                        \
    while (! map->empty()) {                    \
        map->erase(map->begin());               \
    }                                           \
        delete map;

#define FrameType       (1L <<  0)
#define WindowType      (1L <<  1)
#define TitleType       (1L <<  2)
#define LabelType       (1L <<  3)
#define ButtonType      (1L <<  4)
#define HandleType      (1L <<  5)
#define LGripType       (1L <<  6)
#define RGripType       (1L <<  7)
#define RootType        (1L <<  8)
#define WEdgeType       (1L <<  9)
#define EEdgeType       (1L << 10)
#define NEdgeType       (1L << 11)
#define SEdgeType       (1L << 12)
#define MenuTitleType   (1L << 13)
#define MenuItemType    (1L << 14)
#define MenuCBItemType  (1L << 15)
#define MenuSubType     (1L << 16)
#define DockHandlerType (1L << 17)
#define DockAppType     (1L << 18)
#define MenuType        (1L << 19)
#define SystrayType     (1L << 20)
#define LeftEdgeType    (1L << 21)
#define RightEdgeType   (1L << 22)

enum {
    MoveType,
    MoveOpaqueType,
    ResizeType,
    ResizeOpaqueType,
    EndMoveResizeType
};

// switch to these
#define MaxCBoxType     (1L <<  1)
#define MinCBoxType     (1L <<  2)
#define ShadeCBoxType   (1L <<  3)
#define StickCBoxType   (1L <<  4)
#define TitleCBoxType   (1L <<  5)
#define HandleCBoxType  (1L <<  6)
#define BorderCBoxType  (1L <<  7)
#define AllCBoxType     (1L <<  8)
#define AOTCBoxType     (1L <<  9)
#define AABCBoxType     (1L << 10)
#define FsCBoxType      (1L << 11)
#define CloseCBoxType   (1L << 12)
#define HMaxCBoxType    (1L << 13)
#define VMaxCBoxType    (1L << 14)

//#define MaxCBoxType     1
//#define MinCBoxType     2
//#define ShadeCBoxType   3
//#define StickCBoxType   4
//#define TitleCBoxType   5
//#define HandleCBoxType  6
//#define BorderCBoxType  7
//#define AllCBoxType     8
//#define AOTCBoxType     9
//#define AABCBoxType    10
//#define FsCBoxType     11
//#define CloseCBoxType  12

struct ltcase {
    inline bool operator()(std::string a, std::string b) const {
        return strcasecmp(a.c_str(), b.c_str()) < 0;
    }
};

struct ltcase_cstr {
    inline bool operator()(const char* a, const char* b) const {
        return strcasecmp(a, b) < 0;
    }
};


typedef KaScreen* KaScreen_Ptr;

#ifdef XINERAMA
#include "Xinerama.hh"
#endif

#include "Functions.hh"

class Kahakai {
public:
    Kahakai(char **, struct kaoptions *);
    virtual ~Kahakai(void);

    void    RunEventLoop();
    static  Kahakai* Instance();

    WindowObject *FindWin(Window, int);

    struct kaoptions *options;
    Display *display;
    ResourceHandler *rh;
    EventHandler *eh;
    NetHandler *net;
    Timer *timer;
    KaScreen* script_current_screen;
    Cursor session_cursor, move_cursor;
    Cursor resizenw_cursor, resizene_cursor, resizesw_cursor, resizese_cursor;
    Cursor resizew_cursor, resizee_cursor;
    unsigned long double_click, screenmask;
    char *pathenv;
    bool wmerr;

    std::map<Window, WindowObject *> window_table;
    std::vector<KaScreen *> kascreen_list;

    void registerWindowFunction(const char* name, WindowFunction* function) {
        windowFunctions[name] = function;
    }

    void registerScreenFunction(const char* name, ScreenFunction* function) {
        screenFunctions[name] = function;        
        
        // ScreenFunction inherits from WindowFunction and MenuFunction
        windowFunctions[name] = function;
        menuFunctions[name]   = function;
    }

    void registerMenuFunction(const char* name, MenuFunction*   function) {
        menuFunctions[name] = function;
    }

    WindowFunction* getWindowFunction(const char* name) {
        return windowFunctions.count(name) > 0 ? windowFunctions[name] : NULL;
    }

    ScreenFunction* getScreenFunction(const char* name) {
        return screenFunctions.count(name) > 0 ? screenFunctions[name] : NULL;
    }

    MenuFunction* getMenuFunction(const char* name) {
        return menuFunctions.count(name) > 0 ? menuFunctions[name] : NULL;
    }

    int shape, shape_event;

# ifdef XINERAMA
    Xinerama *xinerama;
# endif // XINERAMA

# ifdef XRANDR
    int randr, randr_event;
# endif // XRANDR


private:
    static Kahakai* _instance;

    std::map<const char*, WindowFunction*, ltcase>  windowFunctions;
    std::map<const char*, MenuFunction*,   ltcase>    menuFunctions;
    std::map<const char*, ScreenFunction*, ltcase>  screenFunctions;
};

bool validatedrawable(Drawable, unsigned int * = NULL, unsigned int * = NULL);
const bool validateclient_mapped(Window);
void kaexec(const char *, char *);
void kaexec_if_exists(const char *, char *);
int xerrorhandler(Display *, XErrorEvent *);
int wmrunningerror(Display *, XErrorEvent *);
void signalhandler(int);
void restart(char *);
void quit(int);
char **commandline_to_argv(char *, char **);

std::string expand(std::string, KaWindow *);
std::string trim(std::string str);
std::string replaceAll(std::string str,
                       std::string sub, std::string replacement);
std::string lower(std::string a);
int msDifference(struct timeval first, struct timeval second);

bool viewportStrToXY(std::string viewport_str, int * ret_x, int * ret_y);
int desktopStrToNum(std::string desktop_str);
unsigned int desktopMaskStrToMask(const char * mask, unsigned int oldmask,
                                  unsigned int allowed);

// Credit: pekwm
inline bool overlaps(int x1, int x2, int t1, int t2) {
    if (x1 > t1) {
        if (x1 < t2)
            return true;
    } else if (x2 > t1) {
        return true;
    }
    return false;
}

/** 
 * @fn intersects(int x11, int y11, int x12, int y12, 
 *                int x21, int y21, int x22, int y22)
 * 
 * 
 * @return true if rectangle (x11, y11, x12, y12) intersects (x21,
 y21, x22, y22)
*/
inline bool intersects(int x11, int y11, int x12, int y12,
                       int x21, int y21, int x22, int y22)
{
    return (overlaps(x11, x12, x21, x22) &&
            overlaps(y11, y12, y21, y22));
}

template<typename T>
inline std::string to_string(const T t) {
    std::stringstream ss;
    ss << t;
    return ss.str();
}

#endif // __Kahakai_hh
