/* $Id: Logger.hh,v 1.4 2003/11/20 22:34:30 pervago Exp $ */

#ifndef LOG_HH
# define LOG_HH

extern "C" {
# include <time.h>
# include <stdlib.h>
}

# include <iostream>
# include <fstream>

# include "config.h"

# ifndef NO_EXTERN_LOGGER
extern
#endif
void logger_close(void);

class Logger {
public:

    Logger() {

        char log_file[PATH_MAX];
        char st[50];
        struct tm *tm;
        time_t t;

        sprintf(log_file, "%s/.kahakai/log", getenv("HOME"));

        _log_file = new std::string(log_file);
        _os = new std::ofstream(log_file, std::ios::app | std::ios::out);

        time(&t);
        tm = localtime(&t);
        strftime(st, 50, "%a %b %d %H:%M:%S %Y", tm);

        *_os << "--- Log opened " << st << std::endl;
        _os->flush();
        atexit(logger_close);
    }

    ~Logger() {

        char st[50];
        struct tm *tm;
        time_t t;

        time(&t);
        tm = localtime(&t);
        strftime(st, 50, "%a %b %d %H:%M:%S %Y", tm);

        *_os << "--- Log closed " << st << std::endl;
        _os->flush();
        delete _os;

        delete _log_file;
    }

    template <typename T> Logger &operator<< (const T &v) {

        std::cerr << v;
        (*_os << v).flush();

        return *this;
    }

    Logger &operator<< (std::ostream &(*m)(std::ostream &)) {

        m(std::cerr);
        m(*_os);

        return *this;
    }

    const std::string log_file() const {
        return *_log_file;
    }

protected:

    std::string *_log_file;
    std::ofstream *_os;
};

inline std::ostream &warn(std::ostream& os) {
  if (os != std::cerr)
    return os;
  return os << PACKAGE << ": warning: ";
}

inline std::ostream &err(std::ostream& os) {
  if (os != std::cerr)
    return os;
  return os << PACKAGE << ": error: ";
}

# ifndef NO_EXTERN_LOGGER
extern Logger &logger;
# endif

#endif // !LOG_HH
