/* $Id: Menu.hh,v 1.26 2004/02/03 01:05:31 hyriand Exp $ */

/**
 * @file   Menu.hh
 * @date   02-Aug-2001 22:40:01
 *
 * @brief Definition of KaMenu, KaMenuItem and TaskSwitcher classes
 *
 * Function declarations and variable definitions for KaMenu, KaMenuItem
 * and TaskSwitcher classes.
 *
 */

#ifndef __Menu_hh
#define __Menu_hh

#include "Types.hh"

#include "Window.hh"

#include <boost/shared_ptr.hpp>

using boost::shared_ptr;
/*
** extern "C" {
** include <X11/Xlib.h>
** }
**
** include <string>
*/

# define MenuExecMask  (1L << 0)
# define MenuSubMask   (1L << 1)
# define MenuWFuncMask (1L << 2)
# define MenuRFuncMask (1L << 3)
# define MenuMFuncMask (1L << 4)

typedef enum {
  NoExtMenuType,
  TaskExtMenuType,
  DesktopExtMenuType,
  MinimizedExtMenuType
} ExtMenuType;

class ImageControl;
class Texture;

class KaMenu : public WindowObject {
public:
    KaMenu(std::string);
    virtual ~KaMenu(void);

    void AddItem(KaMenuItem *);
    void Build(KaScreen *);
    void Shape();
    void Render(void);
    void Map(int, int);
    void ReMap(int, int);
    void Move(int, int, bool = true);
    void Unmap(bool);
    void UnmapSubmenus(bool);
    void UnmapTree(void);
    void CreateOutline(void);
    void DestroyOutline(void);
    void DrawOutline(int, int);
    void Raise(void);
    void Lower(void);
    void FocusItem(int n);
    void FocusFirst(void);

    void clearCache();

    Kahakai *kahakai;
    Display *display;
    KaScreen *kascreen;
    KaImageControl *ic;

    std::list<KaMenuItem *> item_list;

    Window frame, o_west, o_north, o_south, o_east;
    int x, y, width, height, bullet_width, cb_width, extra_width;
    bool mapped, built, has_focus, dynamic, dynamic_root, ignore, db,
        cb_db_upd, need_delete;
    std::string name;
    Pixmap pbackframe, ptitle, philite, psub, psubhilite;
    unsigned long backframe_pixel, title_pixel, hilite_pixel,
        sub_pixel, subhilite_pixel;
    KaMenu *root_menu;
    KaMenuItem *root_item;

    ExtMenuType ext_type;
    int ftype;
    Window wf;
    KaScreen *rf;
    KaMenuItem *mf;

    Pixmap pixmap;
    bool render_if_transparency;

    bool dirty;

    bool iconsPresent;

    ImageControl* control;
private:
    int f_height, t_height, s_height;
    shared_ptr<Texture> im_texture;

    void createWindows();
    void determineItemDimensions();
};

#endif // __Menu_hh
