/* $Id: MenuItem.hh,v 1.4 2003/12/26 18:48:21 pervago Exp $ */

/**
 * @file   MenuItem.hh
 * @date   11-Sep-2003
 *
 * @brief Declaration of KaMenuItem class
 *
 * KaMenuItem represents one item in the menu (with optional submenu).
 *
 */

#ifndef __MenuItem_hh
# define __MenuItem_hh

# include "Types.hh"
# include "Window.hh"

#include <boost/shared_ptr.hpp>

using boost::shared_ptr;

/*
extern "C" {
#include <X11/Xlib.h>
}

#include <string>
*/

class KaMenuItem: public WindowObject {
public:
    KaMenuItem(std::string);
    virtual ~KaMenuItem(void);

    void CreateWindow(KaScreen* screen, int y);
    void drawForeground(Drawable);
    void drawBackground(shared_ptr<ImImage> background);
    void refresh();

    void Hilite(void);
    void DeHilite(void);
    void Focus(void);
    void MapSubmenu(XEvent *, KaAction *, bool, bool = false);
    void RemapSubmenu(XEvent *, KaAction *, bool);
    void UnmapMenu(XEvent *, KaAction *, bool);

    void UnLinkMenu(XEvent *, KaAction *);
    inline void MapSubmenu(XEvent *e, KaAction *ac) {
        MapSubmenu(e, ac, false);
    }
    inline void MapSubmenuOnly(XEvent *e, KaAction *ac) {
        MapSubmenu(e, ac, false, true);
    }
    inline void MapSubmenuFocused(XEvent *e, KaAction *ac) {
        MapSubmenu(e, ac, true);
    }
    inline void MapSubmenuFocusedOnly(XEvent *e, KaAction *ac) {
        MapSubmenu(e, ac, true, true);
    }
    inline void RemapSubmenu(XEvent *e, KaAction *ac) {
        RemapSubmenu(e, ac, false);
    }
    inline void RemapSubmenuFocused(XEvent *e, KaAction *ac) {
        RemapSubmenu(e, ac, true);
    }
    inline void UnmapMenu(XEvent *e, KaAction *ka) {
        UnmapMenu(e, ka, false);
    }
    inline void UnmapMenuFocus(XEvent *e, KaAction *ka) {
        UnmapMenu(e, ka, true);
    }
    void Exec(XEvent *, KaAction *);
    void Func(XEvent *, KaAction *);
    void Move(XEvent *, KaAction *);
    void MoveOpaque(XEvent *, KaAction *);
    void EndMoveResize(XEvent *, KaAction *);
    void Focus(XEvent *, KaAction *);
    void UnmapSubmenus(XEvent *, KaAction *);
    void UnmapTree(XEvent *, KaAction *);
    void Raise(XEvent *, KaAction *);
    void Lower(XEvent *, KaAction *);
    void NextItem(XEvent *, KaAction *);
    void PreviousItem(XEvent *, KaAction *);
    inline void Nop(XEvent *, KaAction *) {}

    void EvAct(XEvent *, EventDetail *, std::vector<KaAction *> *);
    void UpdateCBox(void);
    int ExpandAll(KaWindow *);

    int func_mask, func_mask1, func_mask2, height, width, dy, realheight,
                           cb, cb_y, cb_width, cb_width1, cb_width2;
    bool hilited, move_resize, in_window, sdyn, sdyn1, sdyn2, db;
    std::string label, exec, param, sub;
    std::string label1, exec1, param1, sub1;
    std::string label2, exec2, param2, sub2;
    std::string e_label, e_label1, e_label2, e_sub, e_sub1, e_sub2;
    std::string cbox;
    WindowFunction *wfunc, *wfunc1, *wfunc2;
    MenuFunction   *mfunc, *mfunc1, *mfunc2;
    ScreenFunction *rfunc, *rfunc1, *rfunc2;
    KaMenu *menu;
    KaMenu *submenu, *submenu1, *submenu2;
    Window wf;
    shared_ptr<Texture> texture;
    KaFont *kafont_cb;
    XftDraw *xftdraw;
    Pixmap pixmap;
    shared_ptr<ImImage> icon;
private:
    void drawLabel(Pixmap pixmap, int y);
};

#ifdef SWIG
#define SWIG_NOTABSTRACT(cls) %feature("notabstract") cls
#else
#define SWIG_NOTABSTRACT(cls)
#endif

#define EVENT_CLASS(name) SWIG_NOTABSTRACT(MenuEv##name); class MenuEv##name: public MenuFunction { public: MenuEv##name() {}; void operator()(KaMenuItem*, XEvent*, KaAction*); };

#define EVENT_CLASS_INLINE(name, body)  SWIG_NOTABSTRACT(MenuEv##name); class MenuEv##name: public MenuFunction  { public: MenuEv##name() {}; void operator()(KaMenuItem* self, XEvent* e, KaAction* ac) { body; } };

#define WRAP(name) EVENT_CLASS_INLINE(name, self->name(e, ac))
#define WRAP_0(name) EVENT_CLASS_INLINE(name, self->name())

class MenuFunction;

WRAP(UnLinkMenu);
WRAP(MapSubmenu);
WRAP(MapSubmenuOnly);
WRAP(MapSubmenuFocused);
WRAP(MapSubmenuFocusedOnly);
WRAP(RemapSubmenu);
WRAP(RemapSubmenuFocused);
WRAP(UnmapMenu);
WRAP(UnmapMenuFocus);
WRAP(Exec);
WRAP(Func);
WRAP(Move);
WRAP(MoveOpaque);
WRAP(EndMoveResize);
WRAP_0(Focus);
WRAP(UnmapSubmenus);
WRAP(UnmapTree);
WRAP(Raise);
WRAP(Lower);
WRAP(NextItem);
WRAP(PreviousItem);
WRAP(Nop);


#undef EVENT_CLASS
#undef EVENT_CLASS_INLINE
#undef WRAP
 
#endif // __MenuItem_hh
