/* $Id: Net.hh,v 1.15 2003/12/26 18:48:21 pervago Exp $ */

/**
 * @file   Net.hh
 * @date   11-Oct-2001 22:36:12
 *
 * @brief Definition of NetHandler class
 *
 * Function declarations and variable definitions for NetHandler class.
 *
 */

#ifndef __NetHandler_hh
# define __NetHandler_hh

# include "Types.hh"

extern "C" {
# include <X11/Xlib.h>
# include <X11/Xutil.h>
# include <X11/Xproto.h>
}

#define MwmHintsDecorations (1L << 1)

#define MwmDecorAll         (1L << 0)
#define MwmDecorBorder      (1L << 1)
#define MwmDecorHandle      (1L << 2)
#define MwmDecorTitle       (1L << 3)
#define MwmDecorMenu        (1L << 4)
#define MwmDecorMinimize    (1L << 5)
#define MwmDecorMaximize    (1L << 6)

#define PropMotifWmHintsElements 3

typedef struct {
    CARD32 flags;
    CARD32 functions;
    CARD32 decorations;
} MwmHints;

#define _NET_WM_STATE_REMOVE 0
#define _NET_WM_STATE_ADD    1
#define _NET_WM_STATE_TOGGLE 2

#define _NET_WM_MOVERESIZE_SIZE_TOPLEFT      0
#define _NET_WM_MOVERESIZE_SIZE_TOP          1
#define _NET_WM_MOVERESIZE_SIZE_TOPRIGHT     2
#define _NET_WM_MOVERESIZE_SIZE_RIGHT        3
#define _NET_WM_MOVERESIZE_SIZE_BOTTOMRIGHT  4
#define _NET_WM_MOVERESIZE_SIZE_BOTTOM       5
#define _NET_WM_MOVERESIZE_SIZE_BOTTOMLEFT   6
#define _NET_WM_MOVERESIZE_SIZE_LEFT         7
#define _NET_WM_MOVERESIZE_MOVE              8
#define _NET_WM_MOVERESIZE_SIZE_KEYBOARD     9
#define _NET_WM_MOVERESIZE_MOVE_KEYBOARD    10

class NetHandler {
public:
    NetHandler(Kahakai *);
    virtual ~NetHandler(void);

    void GetWMHints(KaWindow *);
    void GetMWMHints(KaWindow *);
    void GetWMNormalHints(KaWindow *);
    void GetState(KaWindow *);
    void SetState(KaWindow *, int);
    void GetWmState(KaWindow *);
    void SetWmState(KaWindow *);
    void GetVirtualPos(KaWindow *);
    void SetVirtualPos(KaWindow *);
    void GetWmStrut(KaWindow *);
    void GetWmPid(KaWindow *);
    void GetXaName(KaWindow *);
    bool GetNetName(KaWindow *);
    void SetVisibleName(KaWindow *);
    void RemoveVisibleName(KaWindow *);
    void SetDesktop(KaWindow *);
    void SetDesktopMask(KaWindow *);
    void GetDesktop(KaWindow *);

    bool DetermineMappable(Window *);

    void SetSupported(KaScreen *);
    void SetSupportedWMCheck(KaScreen *, Window);
    void SetClientList(KaScreen *);
    void SetClientListStacking(KaScreen *);
    void GetClientListStacking(KaScreen *);
    void SetActiveWindow(KaScreen *, KaWindow *);
    void GetActiveWindow(KaScreen *);
    void GetDesktopViewPort(KaScreen *);
    void SetDesktopViewPort(KaScreen *);
    void SetDesktopGeometry(KaScreen *);
    void SetNumberOfDesktops(KaScreen *);
    void SetCurrentDesktop(KaScreen *);
    void GetCurrentDesktop(KaScreen *);
    void SetDesktopNames(KaScreen *, char *);

    void wXDNDMakeAwareness(Window);
    void wXDNDClearAwareness(Window);

    void SetWorkarea(KaScreen *);
    void DeleteSupported(KaScreen *);

    void GetXRootPMapId(KaScreen *);

    void GetWmType(KaWindow *);

    void SetAllowedActions(KaWindow *);
    void RemoveAllowedActions(KaWindow *);

    bool IsSystrayWindow(Window);
    void SetSystrayWindows(KaScreen *);

    Kahakai *kahakai;
    Display *display;
    XWMHints *wm_hints;
    XSizeHints *size_hints;
    MwmHints *mwm_hints;

    Atom utf8_string;

    Atom mwm_hints_atom;
    Atom wm_state, wm_change_state;

    Atom net_supported, net_supported_wm_check;
    Atom net_client_list, net_client_list_stacking, net_active_window,
        kahakai_net_client_list_stacking_desktop[16];
    Atom net_desktop_viewport, net_desktop_geometry, net_current_desktop,
        net_number_of_desktops, net_desktop_names, net_workarea;
    Atom net_wm_desktop, net_wm_name, net_wm_visible_name, net_wm_strut,
        net_wm_pid;
    Atom net_wm_state, net_wm_state_sticky, net_wm_state_shaded,
        net_wm_state_hidden, net_wm_maximized_vert, net_wm_maximized_horz,
        net_wm_state_above, net_wm_state_below, net_wm_state_stays_on_top,
        net_wm_state_stays_at_bottom, net_wm_state_fullscreen,
        net_wm_state_skip_taskbar;
    Atom net_wm_allowed_actions, net_wm_action_move, net_wm_action_resize,
        net_wm_action_minimize, net_wm_action_shade, net_wm_action_stick,
        net_wm_action_maximize_horz, net_wm_action_maximize_vert,
        net_wm_action_fullscreen, net_wm_action_change_desktop,
        net_wm_action_close;
    Atom net_wm_window_type, net_wm_window_type_desktop,
        net_wm_window_type_dock, net_wm_window_type_toolbar,
        net_wm_window_type_menu, net_wm_window_type_splash,
        net_wm_window_type_dialog, net_wm_window_type_utility,
        net_wm_window_type_normal;
    Atom net_close_window, net_moveresize_window, net_wm_moveresize;
    Atom net_startup_info_begin, net_startup_info;
    
    Atom net_wm_icon;

    Atom kahakai_net_wm_state_decor, kahakai_net_wm_state_decortitle,
        kahakai_net_wm_state_decorhandle,
        kahakai_net_wm_state_decorborder;
    Atom kahakai_net_maximized_restore, kahakai_net_virtual_pos,
        kahakai_net_wm_desktop_mask;
    Atom kahakai_net_restart, kahakai_net_shutdown;

    Atom kahakai_net_wm_maximize_restore_handlebar;

    Atom xdndaware, xdndenter, xdndleave;

    Atom kde_net_wm_system_tray_window_for, kde_net_system_tray_windows;

    Atom xrootpmap_id;
private:
    XEvent event;

    int real_format;
    Atom real_type;
    unsigned long items_read, items_left;
};

#endif // __NetHandler_hh
