/* $Id: NewStyleLoader.cc,v 1.22 2004/02/03 01:05:31 hyriand Exp $ */

/**
 * @file   NewStyleLoader.cc
 * @date   12 Jul 2003
 *
 * @brief Implementation of the style parser for the new (.kastyle) styles.
 *
 */

#include "NewStyleLoader.hh"
#include "NewStyleParser.hh"
#include "Screen.hh"
#include "Resources.hh"
#include "ExpressionEvaluator.hh"
#include "Logger.hh"
#include "Imlib2pp.hh"
#include "Texture.hh"

#include <string>
#include <iostream>
#include <fstream>
#include <vector>
#include <utility>
#include <sstream>
#include <map>
#include <stdlib.h>

using std::max;
using std::map;
using std::cout;
using std::endl;
using std::pair;
using std::string;
using std::vector;
using std::ifstream;
using std::stringstream;
using std::make_pair;

NewStyleLoader::NewStyleLoader() {
}

NewStyleLoader::~NewStyleLoader() {
}

void NewStyleLoader::loadStyle(KaScreen* kascreen)
{
    string value;
    int screen = kascreen->screen_number;
    WindowStyle *wstyle = &kascreen->style.window;
    MenuStyle   *mstyle = &kascreen->style.menu;
    KaImageControl *ic = kascreen->ic;

    this->kascreen = kascreen;

    parser.parse(kascreen->config.style_file);

    display  = kascreen->kahakai->display;

    int slen = strlen(kascreen->config.style_file) - 1;
    for (; slen >= 1 && kascreen->config.style_file[slen] != '/'; slen--);
    kascreen->config.style_file[slen] = '\0';

    readFonts();

    readPadding("window.title.buttonPadding", wstyle->buttonPadding);

    readWindowState("window.focused", wstyle->focused);
    readWindowState("window.unfocused", wstyle->unfocused);

    getIntegerValue("window.leftEdge.width",
                    wstyle->leftEdgeWidth, 0, 100, 0);
    getIntegerValue("window.rightEdge.width",
                    wstyle->rightEdgeWidth, 0, 100, 0);

    wstyle->justify = readJustify("window.justify");

    getIntegerValue("window.title.width", wstyle->titleBarWidth, 0, 1000);
    getIntegerValue("window.title.percentWidth", 
                    wstyle->titleBarWidthPercent, 0, 100);
    if (getValue("window.title.fitText", value)) {
        if (! strncasecmp("true", value.c_str(), value.size())) {
            wstyle->fitText = true;
        } else {
            wstyle->fitText = false;
        }
    }

    readRoundedCorners("window.roundedCorners", 
                       wstyle->roundedTopLeft,
                       wstyle->roundedTopRight, 
                       wstyle->roundedBottomLeft, 
                       wstyle->roundedBottomRight);
    readRoundedCorners("menu.roundedCorners", 
                       mstyle->roundedTopLeft,
                       mstyle->roundedTopRight, 
                       mstyle->roundedBottomLeft, 
                       mstyle->roundedBottomRight);

    readTexture(kascreen, "menu.frame.style", mstyle->frame.texture);
    readTexture(kascreen, "menu.title.style", mstyle->title.texture);
    readTexture(kascreen, "menu.hilite.style", mstyle->frame.highlightTexture);

    readDatabaseColor("menu.frame.text.color",
                      &mstyle->frame.color, 
                      BlackPixel(display, screen), ic);
    readDatabaseColor("menu.frame.text.shadow.color",
                      &mstyle->frame.shadowColor, 
                      BlackPixel(display, screen), ic);
    
    readDatabaseColor("menu.hilite.text.color", 
                      &mstyle->frame.highlightColor, 
                      BlackPixel(display, screen),
                      ic);
    readDatabaseColor("menu.hilite.text.shadow.color",
                      &mstyle->frame.highlightShadowColor, 
                      BlackPixel(display, screen),
                      ic);
    
    readDatabaseColor("menu.title.text.color", 
                      &mstyle->title.color, 
                      BlackPixel(display, screen), ic);
    readDatabaseColor("menu.title.text.shadow.color",
                      &mstyle->title.shadowColor, 
                      BlackPixel(display, screen), ic);

    getIntegerValue("menu.frame.text.shadow.offsetX", 
                    mstyle->frame.font.shadow_off_x, -10, 10);
    getIntegerValue("menu.frame.text.shadow.offsetY", 
                    mstyle->frame.font.shadow_off_y, -10, 10);

    getIntegerValue("menu.hilite.shadow.offsetX", 
                    mstyle->frame.highlightFont.shadow_off_x, -10, 10);
    getIntegerValue("menu.hilite.shadow.offsetY", 
                    mstyle->frame.highlightFont.shadow_off_y, -10, 10);

    getIntegerValue("menu.title.text.shadow.offsetX", 
                    mstyle->title.font.shadow_off_x, -10, 10);
    getIntegerValue("menu.title.text.shadow.offsetY", 
                    mstyle->title.font.shadow_off_y, -10, 10);

    mstyle->frame.justify = mstyle->title.justify = 
        readJustify("menu.justify");

    mstyle->frame.justify = readJustify("menu.frame.justify",
                                        mstyle->frame.justify);
    mstyle->title.justify = readJustify("menu.title.justify",
                                        mstyle->title.justify);

    char *look_tmp;
    unsigned int ch;
    if (getValue("menu.bullet.look", value)) {
        if (sscanf(value.c_str(), "'%u'", &ch) != 1) {
            mstyle->bullet.label = value;
        } else {
            look_tmp = new char[2];
            sprintf(look_tmp, "%c", ch);
            mstyle->bullet.label = look_tmp;
        }
    } else {
        look_tmp = new char[2];
        sprintf(look_tmp, ">");
        mstyle->bullet.label = look_tmp;
    }

    if (getValue("menu.checkbox.true.look", value)) {
        if (sscanf(value.c_str(), "'%u'", &ch) != 1) {
            mstyle->checkboxTrue.label = value;
        } else {
            look_tmp = new char[2];
            sprintf(look_tmp, "%c", ch);
            mstyle->checkboxTrue.label = look_tmp;
        }
    } else
        mstyle->checkboxTrue.label = "[x]";

    if (getValue("menu.checkbox.false.look", value)) {
        if (sscanf(value.c_str(), "'%u'", &ch) != 1) {
            mstyle->checkboxFalse.label = value;
        } else {
            look_tmp = new char[2];
            sprintf(look_tmp, "%c", ch);
            mstyle->checkboxFalse.label = look_tmp;
        }
    } else
        mstyle->checkboxFalse.label = "[ ]";

    
    readDatabaseColor("borderColor",
                      &wstyle->borderColor, BlackPixel(display, screen), ic);
    mstyle->borderColor = wstyle->borderColor;

    readDatabaseColor("outlineColor",
                      &wstyle->outlineColor, WhitePixel(display, screen),
                      ic);
    mstyle->borderColor = wstyle->borderColor;

    getIntegerValue("window.handleThickness", wstyle->handleThickness, 
                    0, 6, 6);
    getIntegerValue("window.borderThickness", wstyle->borderThickness, 
                    0, 100, 1);
    
    // Make edges large enough to be able to draw borders
    wstyle->leftEdgeWidth = max(wstyle->leftEdgeWidth, 
                                wstyle->borderThickness);

    wstyle->rightEdgeWidth = max(wstyle->rightEdgeWidth, 
                                 wstyle->borderThickness);

    mstyle->borderThickness = wstyle->borderThickness;

    if (getValue("menu.borderThickness", value))
        sscanf(value.c_str(), "%u", &mstyle->borderThickness);
    
    getIntegerValue("window.title.height", wstyle->titleHeight, 0, 100);
    getIntegerValue("menu.title.height",   mstyle->titleHeight, 0, 100);

    mstyle->itemHeight = mstyle->titleHeight;
    getIntegerValue(string("menu.item.height"), mstyle->itemHeight,  0, 50);
    
    if (getValue("rootCommand", value) && kascreen->config.rootCommand == "")
        kaexec(value.c_str(), kascreen->displaystring);

    int num = 0;
    char rc_name[50];
    vector<DockStyle>::iterator dit = kascreen->style.dockStyles.begin();
    for (; dit != kascreen->style.dockStyles.end(); ++dit, ++num) {
        dit->style.borderColor = wstyle->borderColor;
        dit->style.borderThickness = wstyle->borderThickness;

        sprintf(rc_name, "dockappholder.dock%d.frame.style", num);
        readTexture(kascreen, rc_name, dit->style.texture);

        sprintf(rc_name, "dockappholder.dock%d.borderThickness", num);
        if (getValue(rc_name, value)) {
            if (sscanf(value.c_str(), "%u", &dit->style.borderThickness) != 1)
                dit->style.borderThickness = wstyle->borderThickness;
        }
        sprintf(rc_name, "dockappholder.dock%d.borderColor", num);
        if (getValue(rc_name, value))
            readDatabaseColor(rc_name, &dit->style.borderColor,
                              BlackPixel(display, screen), ic);
    }

    TexturePtr tf_tmp, tu_tmp, tp_tmp;
    KaColor cf_tmp, cu_tmp, cp_tmp;

    readTexture(kascreen, "window.buttons.focused.style",
                tf_tmp);
    readTexture(kascreen, "window.buttons.unfocused.style",
                tu_tmp);
    readTexture(kascreen, "window.buttons.pressed.style",
                tp_tmp);
    
    readDatabaseColor("window.buttons.focused.picColor",
                      &cf_tmp, BlackPixel(display, screen), ic);
    readDatabaseColor("window.buttons.unfocused.picColor",
                      &cu_tmp, WhitePixel(display, screen), ic);
    readDatabaseColor("window.buttons.pressed.picColor",
                      &cp_tmp, cf_tmp.getPixel(), ic);

    vector<ButtonStyle>& buttonstyles = wstyle->buttonStyles;

    ButtonStyle b;

    vector<ButtonStyle>::iterator bit = buttonstyles.begin();
    for (; bit != buttonstyles.end(); ++bit) {
        bit->fg = true;
        bit->x = 0;
        bit->focused.texture = bit->focusedActive.texture = tf_tmp;
        bit->focused.color = bit->focusedActive.color = cf_tmp;
        bit->unfocused.texture = bit->unfocusedActive.texture = tu_tmp;
        bit->unfocused.color = bit->unfocusedActive.color = cu_tmp;
        bit->pressed.texture = bit->pressedActive.texture = tp_tmp;
        bit->pressed.color   = bit->pressedActive.color = cp_tmp;
    }

    map<string, int> types;
    types["shade"]    = ShadeCBoxType;
    types["close"]    = CloseCBoxType;
    types["maximize"] = MaxCBoxType;
    types["minimize"] = MinCBoxType;

    bool found = true;
    TreeNode *nodeWindow, *nodeButtons;
    if ((nodeWindow = parser.getRoot()->child("window")) &&
        (nodeButtons = nodeWindow->child("buttons"))) {        
        
        for (unsigned int i=0;i<nodeButtons->subBlocks.size();i++) {
            TreeNode* node = nodeButtons->subBlocks[i];

            string typeName = node->name;
            size_t digitPos = node->name.find_last_of("0123456789");
            if (digitPos != string::npos && digitPos > 1) {
                typeName = node->name.substr(0, digitPos);                
            }

            if (types.count(typeName) > 0) {
                b.id = i;
                b.autoplace = EastType;
                b.cb = b.x = 0;
                b.fg = true;
                b.focused.texture = tf_tmp;
                b.focused.color = cf_tmp;
                b.unfocused.texture = tu_tmp;
                b.unfocused.color = cu_tmp;
                b.pressed.texture = tp_tmp;
                b.pressed.color = cp_tmp;
                b.action = typeName;
                
                string prefix = string("window.buttons.") + node->name;
                b.cb = types[typeName];
                
                if (getValue(prefix + ".foreground", value)) {
                    if (! strncasecmp("true", value.c_str(), value.size())) 
                        b.fg = true;
                    else 
                        b.fg = false;
                    
                    found = true;
                }
                
                if (getValue(prefix + ".autoplace", value)) {
                    if (! strncasecmp("Left", value.c_str(), value.size()))
                        b.autoplace = WestType;
                    else if(! strncasecmp("False", value.c_str(), value.size()))
                        b.autoplace = 0;
                    else b.autoplace = EastType;
                    found = true;            
                }            
                
                if (getValue(prefix + ".position", value)) {
                    if (! sscanf(value.c_str(), "%d", &b.x)) {
                        b.autoplace = EastType;
                    }
                    else if (b.x != 0) b.autoplace = 0;
                    found = true;
                }

                readButtonPart(kascreen, b.focused, 
                               prefix + ".false.focused");
                readButtonPart(kascreen, b.unfocused, 
                               prefix + ".false.unfocused");
                readButtonPart(kascreen, b.pressed, 
                               prefix + ".false.pressed");

                b.focusedActive.texture = b.focused.texture;
                b.focusedActive.color = b.focused.color;
                b.unfocusedActive.texture = b.unfocused.texture;
                b.unfocusedActive.color = b.unfocused.color;
                b.pressedActive.texture = b.pressed.texture;
                b.pressedActive.color   = b.pressed.color;

                readButtonPart(kascreen, b.focusedActive,   
                               prefix + ".true.focused");
                readButtonPart(kascreen, b.unfocusedActive, 
                               prefix + ".true.unfocused");
                readButtonPart(kascreen, b.pressedActive,   
                               prefix + ".true.pressed");

                if (found) buttonstyles.push_back(b);                
            }
        }
    }
    wstyle->numButtons = buttonstyles.size();
}

bool NewStyleLoader::readButtonPart(KaScreen* kascreen, 
                                    ButtonStateStyle& style, string prefix)
{
    string value;
    KaImageControl *ic = kascreen->ic;
    int screen = kascreen->screen_number;

    bool found = false;
    if (getValue(prefix + ".style", value)) {
        readTexture(kascreen, prefix + ".style", style.texture);
        found = true;
    }        
    if (getValue(prefix + ".picColor", value)) {
        readDatabaseColor(prefix + ".picColor", 
                          &style.color,
                          BlackPixel(display, screen), ic);
        found = true;
    }
    return found;
}

void NewStyleLoader::readWindowState(const std::string& prefix,
                                     WindowStateStyle& style) {    

    int screen = kascreen->screen_number;
    KaImageControl *ic = kascreen->ic;

    readTexture(kascreen, prefix + ".title.style",  style.titleTexture);
    readTexture(kascreen, prefix + ".label.style",  style.labelTexture);
    readTexture(kascreen, prefix + ".handle.style", style.handleTexture);

    readTexture(kascreen, prefix + ".grip.style",   
                style.leftGripTexture);
    style.rightGripTexture = style.leftGripTexture;

    string dummy;
    if (getValue(prefix + ".leftGrip.style", dummy)) {
        readTexture(kascreen, prefix + ".leftGrip.style",
                    style.leftGripTexture);
    }

    if (getValue(prefix + ".rightGrip.style", dummy)) {
        readTexture(kascreen, prefix + ".rightGrip.style",
                    style.rightGripTexture);
    }

    readTexture(kascreen, prefix + ".leftEdge.style", style.leftEdgeTexture);
    readTexture(kascreen, prefix + ".rightEdge.style", style.rightEdgeTexture);

    readDatabaseColor(prefix + ".label.text.color",
                      &style.labelColor, 
                      BlackPixel(display, screen), ic);
    readDatabaseColor(prefix + ".label.text.shadow.color",
                      &style.labelShadowColor,
                      BlackPixel(display, screen), ic);

    getIntegerValue(prefix + ".label.text.shadow.offsetX",
                    style.font.shadow_off_x, -10, 10);
    getIntegerValue(prefix + ".label.text.shadow.offsetY",
                    style.font.shadow_off_y, -10, 10);
}

bool NewStyleLoader::getValue(const string& rname, string& value) {
    return parser.getValue(rname, value);
}

void NewStyleLoader::readTexture(KaScreen* screen, 
                                 const std::string& rname, 
                                 shared_ptr<Texture>& texture) {
    DrawOps ops;
    StyleLoader::readTexture(screen, rname, ops);

    string prefix;

    if (rname.find(".style") != string::npos)
        prefix = rname.substr(0, rname.length() - strlen(".style"));
    else
        prefix = rname;

    TreeNode* drawNode = parser.getRoot()->findNode(prefix + ".draw");
    if (!drawNode) {
        texture = shared_ptr<Texture>(new Texture(ops));
        return;
    }

    screen->style.window.usesDrawing = true;

    vector<TreeDrawLeaf>::iterator it = drawNode->drawLeaves.begin();
    for (; it != drawNode->drawLeaves.end(); ++it) {
        if (it->name == "line") {
            if (it->parameters.size() != 5) {
                logger << warn << 
                    "'line' requires 5 parameters (color, x, y, x2, y2)" 
                       << endl;
                continue;
            }
            
            ops.push_back(shared_ptr<DrawOp>
                          (new DrawOpLine(parseColor(it->parameters[0], 0),
                                          it->parameters[1],
                                          it->parameters[2],
                                          it->parameters[3],
                                          it->parameters[4])));
        } else if (it->name == "fillRect") {
            if (it->parameters.size() != 5) {
                logger << warn << 
                    "'fillRect' requires 5 parameters (color, x, y, width, height)" 
                       << endl;
                continue;
            }
            
            ops.push_back(shared_ptr<DrawOp>
                          (new DrawOpFillRect(parseColor(it->parameters[0], 0),
                                              it->parameters[1],
                                              it->parameters[2],
                                              it->parameters[3],
                                              it->parameters[4])));
        }  else if (it->name == "gradient") {
            if (it->parameters.size() != 7) {
                logger << warn << 
                    "'gradient' requires 7 parameters (color, colorTo, angle, x, y, width, height)" 
                       << endl;
                continue;
            }
            vector<pair<ImColor, int> > range;
            range.push_back(make_pair(parseColor(it->parameters[0]), 1));
            range.push_back(make_pair(parseColor(it->parameters[1]), 1));
            
            ops.push_back(shared_ptr<DrawOp>
                          (new DrawOpGradient(range,
                                              it->parameters[2],
                                              it->parameters[3],
                                              it->parameters[4],
                                              it->parameters[5],
                                              atof(it->parameters[6].c_str()))));
        } else if (it->name == "interlace") {
            if (it->parameters.size() != 1) {
                logger << warn << "'interlace' requires a color parameter" << 
                    endl;
                continue;
            }

            ops.push_back(shared_ptr<DrawOp>
                          (new DrawOpInterlace(64, 128, 
                                               parseColor(it->parameters[0]))));
        } else {
            logger << warn << "Unknown drawing operation " << it->name << 
                    endl;
        }
    }

    texture = shared_ptr<Texture>(new Texture(ops));
}
