/* $Id: NewStyleParser.hh,v 1.2 2003/11/20 22:34:29 pervago Exp $ */

/**
 * @file   NewStyleParser.hh
 * @date   12 Jul 2003
 *
 * @brief Style parser for the new (.kastyle) files.
 *
 */

#ifndef __NewStyleParser_hh
# define __NewStyleParser_hh

# include "Types.hh"

# include <string>
# include <map>
# include <vector>
/*
** include <map>
** include <string>
**
** extern "C" {
** include <X11/Xresource.h>
** }
*/

struct TreeLeaf
{
    std::string name, value;

    TreeLeaf(const TreeLeaf& other): name(other.name), value(other.value) {};
    TreeLeaf(const std::string& _name, const std::string& _value): 
        name(_name), value(_value) {};
};

struct TreeDrawLeaf
{
    typedef std::vector<std::string> Parameters;

    std::string name;
    Parameters parameters;
    
    TreeDrawLeaf(const TreeDrawLeaf& other): name(other.name), 
                                             parameters(other.parameters) {};
    TreeDrawLeaf(const std::string& _name, const Parameters& _parameters):
        name(_name), parameters(_parameters) {};
};

struct TreeNode
{
    TreeNode* parent;

    TreeNode() {};

    TreeNode(const TreeNode& other): 
        parent(other.parent), name(other.name), subBlocks(other.subBlocks),
        leaves(other.leaves) {};

    std::string name;
    std::vector<TreeNode*> subBlocks;
    std::vector<TreeLeaf> leaves;
    std::vector<TreeDrawLeaf> drawLeaves;

    TreeNode* findNode(const std::string& name);

    TreeNode* child(const std::string& name) {
        for (unsigned int i=0;i<subBlocks.size();i++) {
            if (subBlocks[i]->name == name) return subBlocks[i];
        }
        return NULL;
    }
};
class NewStyleParser
{
 public:
    NewStyleParser();
    virtual ~NewStyleParser();

    TreeNode* getRoot() {
        return root;
    }

    bool getValue(const std::string& rname, std::string& value);
    void parse(const char* filename);
 private:
    void processBlock(const std::string& line, int indent);
    void processAssignment(const std::string& line);
    void processDrawOperation(const std::string& line);

    void includeBlock(const std::string& blockName, 
                      const std::string& currentBlock);

    std::string processValue(const std::string& value);


    std::map<std::string, std::string> defs;
    TreeNode* root;
    
    std::vector<std::pair<int, std::string> > blocks;
    int last_block_indent;
    TreeNode* currentBlockNode;

    bool drawMode;
};

#endif
