/* $Id: PythonScript.cc,v 1.17 2003/12/03 05:46:22 mackstann Exp $ */

/**
 * @file   PythonScript.cc
 * @date   31-May-2003 22:08:02
 *
 * @brief Put brief explanation here
 *
 * Put a long explanation here.
 *
 */

#include "PythonScript.hh"
#include "ScriptFactory.hh"
#include "Logger.hh"

#include <Python.h>

#include <dlfcn.h>
#include <unistd.h>

#include <iostream>
#include <cstring>
#include <cerrno>
#include <string>

using namespace std;

extern "C" {
    extern void init_kahakai();
}

/*
 * file static
 */

// c++ doesnt have inline list-construction syntax
static list<string> makeSuffixes() {
    list<string> suffixes;
    suffixes.push_back("py");
    return suffixes;
}

static Script * constructor(const string &filename) {
    return new PythonScript(filename);
}

/*
 * static initialization
 */

int PythonScript::_count = 0;

int PythonScript::_staticInit = doStaticInit();
int PythonScript::doStaticInit() {

    bool success = true;

    // register our Constructor with ScriptFactory
    Constructor ctor = constructor;
    success = success && ScriptFactory::registerScript(makeSuffixes(), ctor);

    // check success
    //if (success)
        // uh... this was segfaulting.  very interesting.
        //logger << "Loaded PythonScript" << endl;
    //else
    if (!success)
        // kill everything
        throw "Failed to load PythonScript";

    // return value arbitrary
    return 0;
}

/*
 * public
 */

PythonScript::PythonScript(const string &filename)
    : Script(makeSuffixes(), filename)
{
    // if this is the first script, load up the interpreter
    if (_count++ == 0) {
        envInitialize();
    }
}

PythonScript::~PythonScript() {

    // if this was the last script, shutdown the interpreter
    if (--_count == 0) {
        envFinalize();
    }
}

bool PythonScript::run() {

    try {

        // this should have been taken care of at construction
        assert(checkSuffix(getFilename()));

        // try to open the file (it should have existed at construction)
        char *c_path = const_cast<char *>(getFilename().c_str());
        FILE *f = fopen(c_path, "r");

        // did the file open?
        if (f) {
            int retval = PyRun_SimpleFile(f, c_path);
            if (retval != 0 || PyErr_Occurred()) {
                return false;
            }
            fclose(f);
            return true;

        // ...nope
        } else {
            throw string("Failed to open file: " + getFilename() + 
                         string(": ") + strerror(errno));
        }

    // failed to run for some reason
    } catch (string e) {
        logger << err << "ERROR: " << string(e) << ", aborting run" << endl;
        return false;
    }

    // ran successfully
    return true;
}

/*
 * protected
 */

/*
 * private
 */

typedef void (*void_func_t)();

void PythonScript::envInitialize() {
    Py_Initialize();

    void* handle = dlopen(PKGLIBDIR "/libkahakai_swig_python.so", RTLD_LAZY);
    if (!handle) {
        printf("couldn't open libkahakai_swig_python.so: %s\n", dlerror());
        exit(1);
    }

    void_func_t _init_kahakai;

    _init_kahakai = (void_func_t)dlsym(handle, "init_kahakai");

    if (!_init_kahakai) {
        printf("couldn't get init_kahakai function: %s\n", dlerror());
        exit(1);
    }

    _init_kahakai();

    // Add script directories to Python module path
    string code = "import sys\n";

    list<string> script_dirs = scriptDirs();
    list<string>::reverse_iterator it = script_dirs.rbegin();
    for (; it != script_dirs.rend(); it++) {
        code += "sys.path.insert(0, r'" +  *it + "')\n";
    }

    code += "del sys\n";

    PyRun_SimpleString(const_cast<char*>(code.c_str()));
}

void PythonScript::envFinalize() {
    Py_Finalize();
}
