/* $Id: Resources.hh,v 1.26 2004/02/02 21:47:25 brailsmt Exp $ */

/**
 * @file   Resources.hh
 * @date   18-Jul-2001 00:31:06
 *
 * @brief Definition of ResourceHandler class
 *
 * Function declarations and variable definitions for ResourceHandler class
 *
 */

#ifndef __Resources_hh
# define __Resources_hh

# include "Types.hh"
# include "Kahakai.hh"

extern "C" {
# include <X11/Xresource.h>
}
# include <string>

# define IS_ENV_CHAR(ch) (isalnum(ch) || ch == '_')

# define ACTLISTCLEAR(list) \
    while (! list.empty()) { \
        if (list.back()->exec) \
            delete [] list.back()->exec; \
        if (list.back()->param) \
            delete [] list.back()->param; \
        delete list.back(); \
        list.pop_back(); \
    }

# define ACTLISTPTRCLEAR(list) \
    while (! list->empty()) { \
        if (list->back()->exec) \
            delete [] list->back()->exec; \
        if (list->back()->param) \
            delete [] list->back()->param; \
        delete list->back(); \
        list->pop_back(); \
    }

/// This is the maximum length that a script directory in a config file is allowed to be.
#define SCRIPT_DIR_MAX_LENGTH	50

enum {
    VerticalDock,
    HorizontalDock,
    AlwaysOnTop,
    AlwaysAtBottom,
    NormalStacking
};

enum {
    NameMatchType,
    ClassMatchType,
    TitleMatchType
};

typedef std::map<std::string, std::string, ltcase> DefinesMap;

class ResourceHandler {
public:
    ResourceHandler(Kahakai *, struct kaoptions *);
    virtual ~ResourceHandler(void);

    void LoadConfig(Kahakai *);
    void LoadConfig(KaScreen *);
    void LoadStyle(KaScreen *);
    void LoadMenus(KaScreen *);
    KaMenu* ParseMenu(KaMenu *, FILE *, KaScreen *);

    char *rc_file, *style_file, *menu_file, *script_config;
    bool rc_forced, style_forced, menu_forced, 
         script_config_forced;
    int linenr;
    
private:
    void CheckConfigDirectory();

    Kahakai *kahakai;
    Display *display;
    XrmDatabase database;
    char *homedir;
};

#define WindowFuncMask (1L << 0)
#define RootFuncMask   (1L << 1)
#define MenuFuncMask   (1L << 2)

char *strtrim(char *);
char *strwithin(char *, char, char, bool = false);
char *environment_expansion(char *s);

#endif // __Resources_hh
