/* $Id: RubyScript.hh,v 1.17 2004/02/04 21:53:28 brailsmt Exp $ */

/// @file   RubyScript.hh
/// @date   05-June-2003 10:49:54
/// @author Michael Brailsford

/* TODO
 * things need to be done with swig before this can be used
 */

#ifndef __RUBYSCRIPT_HH
# define __RUBYSCRIPT_HH

# include <ruby.h>
# undef PACKAGE
# undef PACKAGE_NAME
# undef PACKAGE_STRING
# undef PACKAGE_TARNAME
# undef PACKAGE_VERSION
# undef PACKAGE_BUGREPORT

# include "Types.hh"
# include "Script.hh"
#include "ka_logger.hh"

extern int log_level;

class RubyScript : public Script {
	public:
		/// Create a RubyScript object from a file name.
		/// @param filename The file name
		/// @throw InvalidFileType if filename is not named as a ruby script
		RubyScript(const std::string &filename);

		/// Create a RubyScript object by copying another.
		/// @param that The RubyScript object to copy
		RubyScript(const RubyScript &that);

		/// Destroy this object.
		virtual ~RubyScript();

		/// Execute the ruby script represented by this Script.
		/// @return true if executed successfully, false otherwise
		virtual bool run();

	protected:
		/// Function which can be passed to rb_protect to handle any exception raised by ruby_run(),
		/// @return non-zero if an exception was raised, zero otherwise
		int protected_run(VALUE arg);

		// TODO provide some mechanism by which children can pass-down their
		// suffix list and augment ours? should we worry about children of
		// children of Script?

	private:
		/// Function to initialize interpreter.
		static void initialize();
		/// Function to finalize interpreter.
		static void finalize();

		/// Keep a running count of instances.
		static int _count;

		/// Static initialization hack.
		static int _staticInit;
		/// Static initialization hack.
		static int doStaticInit();
};

#endif
