/* $Id: Screen.hh,v 1.45 2004/02/07 22:43:45 slide_o_mix Exp $ */

/**
 * @file   Screen.hh
 * @date   25-Jul-2001 23:25:51
 *
 * @brief Definition of KaScreen and ScreenEdge classes
 *
 * Function declarations and variable definitions for KaScreen and
 * ScreenEdge classes.
 *
 */

#ifndef __Screen_hh
# define __Screen_hh

# include "Types.hh"

# include "Style.hh"
# include "Functions.hh"
# include "Kahakai.hh"
# include "ImageControl.hh"
# include <sstream>

/*
extern "C" {
#include <X11/Xlib.h>

#include <X11/extensions/Xrender.h>

#ifdef    XINERAMA
#  include <X11/extensions/Xinerama.h>
#endif // XINERAMA

#include <X11/Xft/Xft.h>
}*/

//#include <vector>
//#include <map>

struct WMstrut {
    Window window;
    int left;
    int right;
    int top;
    int bottom;
};

struct Workarea{
    int x;
    int y;
    int width;
    int height;
};

class Desktop {
public:
    inline Desktop(int _number, int w, int h, std::string _name = "") {
        number = _number;
	if( _name.length() == 0 ) {
	    char buf[20];
	    sprintf( buf, "Desktop %d", number + 1 );
	    _name = buf;
	}	    
	name = _name;
        workarea.x = workarea.y = 0;
        workarea.width = w;
        workarea.height = h;
        oldfocus = 0;
    }
    unsigned int number;
    std::string name;
    Workarea workarea;
    Window oldfocus;
    std::list<Window> aot_stacking_list, stacking_list, aab_stacking_list;
};

#define DESKTOP_LOOP(screen) \
    for (std::vector<Desktop *>::iterator __dit = screen->desktop_list.begin(); __dit != screen->desktop_list.end(); ++__dit)

#define DESKTOP_LOOP_MASK(screen, mask) \
    DESKTOP_LOOP(screen) \
        if (mask & (1L << (*__dit)->number))

class SystrayWindow : public WindowObject {
public:
    inline SystrayWindow(Window id, KaScreen *_ws) :
        WindowObject(id, SystrayType) {
        ws = _ws;
    }

    KaScreen *ws;
};

class MReq {
public:
    inline MReq(Window m, KaWindow *w) { mid = m; win = w; }

    Window mid;
    KaWindow *win;
};

const int WestDirection  = 1;
const int EastDirection  = 2;
const int NorthDirection = 3;
const int SouthDirection = 4;

typedef struct {
    char *style_file, *menu_file, *script_config;
    unsigned int virtual_x;
    unsigned int virtual_y;
    unsigned int desktops;
    int colors_per_channel, menu_stacking;
    long unsigned int cache_max;
    bool image_dither, transient_above, db, revert_to_window, moveresize_crosshair;

    bool resist;
    bool snapping;
    int  resistDistance;
    int  snapDistance;

    bool lazy_trans;
 
    std::vector<KaAction *> frameacts, awinacts, pwinacts, titleacts,
                            labelacts, handleacts, rgacts, lgacts,
                            rootacts, weacts, eeacts, neacts, seacts,
                            mtacts, miacts, msacts, mcbacts, lweacts, rweacts;

    std::map<std::string, std::vector<KaAction *> > bacts;

    std::vector<std::string> desktop_names;

    std::string rootCommand;

    time_t menu_file_mtime;
} ScreenConfig;



class KaScreen : public WindowObject {
public:
    KaScreen(Display *, int, Kahakai *);
    virtual ~KaScreen();

    void RaiseWindow(Window);
    void RaiseWindowAllDesktops(KaWindow *);
    void LowerWindow(Window);
    void LowerWindowAllDesktops(KaWindow *);
    void RestackWindows(Window);
    void UpdateCheckboxes(int);
    void ReloadMenu();
    bool CheckMenuMTime();
    KaMenu *GetMenuNamed(std::string);
    KaMenu *CreateDynamicMenu(std::string);
    void MoveViewportTo(int, int);
    void MoveViewport(int, bool wrap = false);
  //    void ScrollViewport(int, bool, KaAction *);
    void MenuMap(XEvent *, KaAction *, bool);
    void MenuRemap(XEvent *, KaAction *, bool);
    void MenuUnmap(XEvent *, KaAction *, bool);
    void UpdateWorkarea();
    void GetWorkareaSize(int *, int *, int *, int *) const;
    inline const Workarea & GetWorkarea() const {
        return current_desktop->workarea;
    }

    void AddDockapp(Window window);
    void GoToDesktop(unsigned int);
    void SetSmartName(KaWindow * win, int smartnum);
    void SmartName(KaWindow *);
    void SmartNameRemove(KaWindow *);
    void ImodAdd(unsigned int mask);

    bool QueryPointer(Window w, 
                      int *root_x_return, int *root_y_return,
                      unsigned int *mask_return);
    bool QueryPointer(Window w, 
                      int *root_x_return, int *root_y_return);

    inline bool QueryPointer(int *rx, int *ry, unsigned int *rmask) {
        return QueryPointer(id, rx, ry, rmask);
    }

    inline bool QueryPointer(int *rx, int *ry) {
        return QueryPointer(id, rx, ry);
    }

#ifdef XRANDR
    void RRUpdate();
#endif // XRANDR
    
    inline void MenuMap(XEvent *e, KaAction *ac) {
        MenuMap(e, ac, false);
    }
    inline void MenuMapFocused(XEvent *e, KaAction *ac) {
        MenuMap(e, ac, true);
    }
    inline void MenuRemap(XEvent *e, KaAction *ac) {
        MenuRemap(e, ac, false);
    }
    inline void MenuRemapFocused(XEvent *e, KaAction *ac) {
        MenuRemap(e, ac, true);
    }
    void ViewportMove(XEvent *, KaAction *);
    void EndMoveResize(XEvent *, KaAction *);
    void Focus(XEvent *, KaAction *);
    inline void MenuUnmap(XEvent *e, KaAction *ka) {
        MenuUnmap(e, ka, false);
    }
    inline void MenuUnmapFocus(XEvent *e, KaAction *ka) {
        MenuUnmap(e, ka, true);
    }
    void Restart(XEvent *, KaAction *);
    void Exit(XEvent *, KaAction *);
    void TaskSwitcher(XEvent *, KaAction *);
    void PreviousTask(XEvent *, KaAction *);
    void NextTask(XEvent *, KaAction *);
    void PointerRelativeWarp(XEvent *, KaAction *);
    void PointerFixedWarp(XEvent *, KaAction *);
    void ViewportRelativeMove(XEvent *, KaAction *);
    void ViewportFixedMove(XEvent *, KaAction *);
    void GoToDesktop(XEvent *, KaAction *);
    void NextDesktop(XEvent *, KaAction *);
    void PreviousDesktop(XEvent *, KaAction *);

    void CreateFonts();

    inline void MoveViewportLeft(XEvent *, KaAction *) {
        MoveViewport(WestDirection);
    }
    inline void MoveViewportRight(XEvent *, KaAction *) {
        MoveViewport(EastDirection);
    }
    inline void MoveViewportUp(XEvent *, KaAction *) {
        MoveViewport(NorthDirection);
    }
    inline void MoveViewportDown(XEvent *, KaAction *) {
        MoveViewport(SouthDirection);
    }

    inline void MoveViewportWrapLeft(XEvent *, KaAction *) {
        MoveViewport(WestDirection, true);
    }
    inline void MoveViewportWrapRight(XEvent *, KaAction *) {
        MoveViewport(EastDirection, true);
    }
    inline void MoveViewportWrapUp(XEvent *, KaAction *) {
        MoveViewport(NorthDirection, true);
    }
    inline void MoveViewportWrapDown(XEvent *, KaAction *) {
        MoveViewport(SouthDirection, true);
    }

    inline void FocusFreezeOn() { focus_freeze = true; }
    inline void FocusFreezeOff() { focus_freeze = false; }
    inline bool FocusFreeze() { return focus_freeze; }

    inline void UserFocusFreezeOn() { user_focus_freeze = true; }
    inline void UserFocusFreezeOff() { user_focus_freeze = false; }
    inline bool UserFocusFreeze() { return user_focus_freeze; }

    void DiscardEnterNotifies();
    void DiscardEventsOfType(int type);

    bool focus_freeze;
    bool user_focus_freeze;

    inline void Nop(XEvent *, KaAction *) {}
    
    void EvAct(XEvent *, EventDetail *, std::vector<KaAction *> *);

    Display *display;
    int screen_number, screen_depth, width, height, v_x, v_y, v_xmax, v_ymax;
    Colormap colormap;
    Visual *visual;
    Kahakai *kahakai;
    NetHandler *net;
    ResourceHandler *rh;
    KaImageControl *ic;
    ImageControl *control;
    Style style;
    ScreenConfig config;
    KaFont default_font;
    XFontStruct *def_font;
    WindowMenu *window_menu;
    ScreenEdgeManager *edgemgr;

    Pixmap fgrip, ugrip;
    Display *pdisplay;    
    
    bool render_extension;
    Pixmap xrootpmap_id;

#ifdef PIXMAP
    Imlib_Context imlib_context;
#endif // PIXMAP

    unsigned long fbutton_pixel, ubutton_pixel, pbutton_pixel, fgrip_pixel,
        ugrip_pixel;
    char displaystring[1024];
    ScreenEdge *west, *east, *north, *south;
    Window wm_check;
    bool focus, shutdown;

    std::vector<Desktop *> desktop_list;
    Desktop *current_desktop;

    std::vector<unsigned int> imod_list;
    unsigned int imod_mask;

    std::vector<KaWindow *> kawindow_list;
    std::list<KaWindow *> kawindow_list_map_order;
    std::vector<KaMenu *> kamenu_list;
    std::list<WMstrut *> strut_list;
    std::list<DockappHandler *> docks;
    std::list<Window> systray_window_list;
    
    std::vector<MReq *> mreqs;

private:
    void LoadScriptConfig();
    void CreateWindows();
    void CreateMenus();
    void CreateVerticalEdges();
    void CreateHorizontalEdges();
    void CreateColors();
    bool raiseWindowInList(Window win, std::list<Window>& stacking, Desktop *desktop);
    void lowerWindowOnDesktop(Window win, Desktop *desktop);

    void CreateXftColor(KaColor *, XftColor *);

    Script *script;
    int move;
};

class ScreenEdge : public WindowObject {
public:
    ScreenEdge(KaScreen *, int, int, int, int, int);
    virtual ~ScreenEdge();

    KaScreen *ka;
};

class ScreenEdgeManager {
public:
    ScreenEdgeManager(KaScreen *);
    ~ScreenEdgeManager();
    ScreenEdge * n, * s, * e, * w;
};

class ScreenFunction;

#ifdef SWIG
#define SWIG_NOTABSTRACT(cls) %feature("notabstract") cls
#else
#define SWIG_NOTABSTRACT(cls)
#endif


#define EVENT_CLASS(name) \
    SWIG_NOTABSTRACT(ScreenEv##name); \
    class ScreenEv##name: public ScreenFunction { \
    public: \
        ScreenEv##name() {}; \
        void operator()(KaScreen*, XEvent*, KaAction*); \
    };

#define EVENT_CLASS_INLINE(name, body) \
    SWIG_NOTABSTRACT(ScreenEv##name); \
    class ScreenEv##name: public ScreenFunction { \
    public: \
        ScreenEv##name() {}; \
        void operator()(KaScreen* self, XEvent* e, KaAction* ac) { \
            body; \
        } \
    };

EVENT_CLASS(GoToDesktop);
EVENT_CLASS(ViewportFixedMove);

EVENT_CLASS_INLINE(MoveViewportLeft,  self->MoveViewport(WestDirection));
EVENT_CLASS_INLINE(MoveViewportRight, self->MoveViewport(EastDirection));
EVENT_CLASS_INLINE(MoveViewportUp,    self->MoveViewport(NorthDirection));
EVENT_CLASS_INLINE(MoveViewportDown,  self->MoveViewport(SouthDirection));

EVENT_CLASS_INLINE(MoveViewportWrapLeft,
                                      self->MoveViewport(WestDirection, true));
EVENT_CLASS_INLINE(MoveViewportWrapRight,
                                      self->MoveViewport(EastDirection, true));
EVENT_CLASS_INLINE(MoveViewportWrapUp,
                                      self->MoveViewport(NorthDirection, true));
EVENT_CLASS_INLINE(MoveViewportWrapDown,
                                      self->MoveViewport(SouthDirection, true));

EVENT_CLASS_INLINE(ReloadMenu,        self->ReloadMenu());
    
EVENT_CLASS_INLINE(MenuMap,           self->MenuMap(e, ac, false));
EVENT_CLASS_INLINE(MenuMapFocused,    self->MenuMap(e, ac, true));

EVENT_CLASS_INLINE(MenuRemap,         self->MenuRemap(e, ac, false));
EVENT_CLASS_INLINE(MenuRemapFocused,  self->MenuRemap(e, ac, true));

EVENT_CLASS_INLINE(MenuUnmap,         self->MenuUnmap(e, ac, false));
EVENT_CLASS_INLINE(MenuUnmapFocus,    self->MenuUnmap(e, ac, true));

EVENT_CLASS_INLINE(ViewportMove,      self->ViewportMove(e, ac));
EVENT_CLASS_INLINE(ViewportRelativeMove, self->ViewportRelativeMove(e, ac));
EVENT_CLASS_INLINE(EndMoveResize,     self->EndMoveResize(e, ac));

EVENT_CLASS_INLINE(Focus,             self->Focus(e, ac));

EVENT_CLASS_INLINE(Restart,           self->Restart(e, ac));
EVENT_CLASS_INLINE(Exit,              self->Exit(e, ac));
EVENT_CLASS_INLINE(TaskSwitcher,      self->TaskSwitcher(e, ac));
EVENT_CLASS_INLINE(NextTask,          self->NextTask(e, ac));   
EVENT_CLASS_INLINE(PreviousTask,      self->PreviousTask(e, ac));
EVENT_CLASS_INLINE(PointerFixedWarp,  self->PointerFixedWarp(e, ac));
EVENT_CLASS_INLINE(PointerRelativeWarp, self->PointerRelativeWarp(e, ac));

EVENT_CLASS_INLINE(NextDesktop,       self->NextDesktop(e, ac));
EVENT_CLASS_INLINE(PreviousDesktop,   self->PreviousDesktop(e, ac));
EVENT_CLASS_INLINE(Nop, ;);

#undef EVENT_CLASS
#undef EVENT_CLASS_INLINE

#endif // __Screen_hh
