// $Id: ScriptFactory.cc,v 1.6 2003/09/11 22:42:21 faugn Exp $

#include "ScriptFactory.hh"
#include "Script.hh"

#include <string>
#include <list>

using std::string;
using std::list;

/*
 * static initialization
 */

// dont initialize here. let it obtain a NULL value by default and be created
// within registerScript. the static initialization of other classes (like
// Script sub-types) might occur before our own
ScriptFactory::MapType *ScriptFactory::_map;

/*
 * public
 */

bool ScriptFactory::registerScript(const list<string> &suffixes,
        Constructor constructor) {

    // this is where _map gets initialized
    if (_map == NULL)
        _map = new MapType();

    // dont allow overwriting of mappings
    list<string>::const_iterator i;
    for (i = suffixes.begin(); i != suffixes.end(); i++) {
        if (_map->count(*i) > 0) {
            // we found a pre-existing mapping
            return false;
        }
    }

    // suffixes wont overwrite any mappings. now create the mappings
    for (i = suffixes.begin(); i != suffixes.end(); i++) {
        (*_map)[*i] = constructor;
    }

    // success
    return true;
}

Script * ScriptFactory::newScript(const string &filename) {

    // make sure we have any data at all
    if (_map == NULL)
        return NULL;

    // dont insert a new value, only check to see if a mapping already exists
    ScriptFactory::MapType::iterator i =_map->find(Script::getSuffix(filename));

    // if it exists and the Constructor is non-null, return a new object
    if (i != _map->end() && i->second != NULL) {
        // construct a new Script sub-type object from the give filename
        return (**i->second)(filename);
    } else {
        return NULL;
    }
}

/*
 * protected
 */

/*
 * private
 */
