// $Id: ScriptFactory.hh,v 1.4 2003/09/11 22:42:21 faugn Exp $

#ifndef __SCRIPTFACTORY_HH
# define __SCRIPTFACTORY_HH

# include "Types.hh"

# include <map>
# include <list>
# include <string>

// a pointer to a method that will create a new object. not a real constructor
typedef Script * (*Constructor) (const std::string &);

/*
 * singleton, abstract factory patterns
 */
class ScriptFactory {

    public:

        /**
         * Register a Script object Constructor with a list of suffixes.
         * Previously registered suffixes will not be overwritten. If any suffix
         * attempts to overwrite a previous registration, no suffix in suffixes
         * will be registered. Thus, either every suffix in suffixes will be
         * registered, or none will.
         * @param suffixes Suffixes that should correspond to this Script type
         * @param constructor A method that will construct a new Script object
         * @return true if all suffixes were registered, false if none were
         */
        static bool registerScript(const std::list<std::string> &suffixes,
                Constructor constructor);

        /**
         * Create a Script object for a given file name
         * @param filename The file name of the script
         * @return A prototypical Script sub-type object
         * @return A Script sub-type object able to handle filename
         */
        static Script * newScript(const std::string &filename);

    private:

        // to be reused
        typedef std::map<std::string, Constructor> MapType;

        // our mappings from suffix -> Script objects
        static MapType *_map;

};

#endif
