/* $Id: SimpleMatch.cc,v 1.4 2003/12/01 00:18:17 pervago Exp $ */

/**
 * @file   SimpleMatch.cc
 * @date   3 Nov 2003
 *
 * @brief Implementation of SimpleMatch class.
 *
 * Simple matching of patterns using ^ to specify the beginning of a string,
 * and $ to specify the end.
 *
 */

#include "SimpleMatch.hh"
#include "Kahakai.hh"

#include <iostream>
#include <string>

using std::cerr;
using std::cout;
using std::endl;
using std::string;

/**
 * @fn    SimpleMatch(void)
 * @brief Constructor for SimpleMatch class
 *
 * @param pattern Pattern to match against. Use '^pat' to search for
 * 'pat' at beginning of a string, 'pat$' to search for 'pat' at end
 * of the string, '^pat$' to search for exact match to 'pat', just
 * 'pat' to search for substring 'pat'.
 */
SimpleMatch::SimpleMatch(string the_pattern) {
    pattern = trim(the_pattern);

    match_front = match_back = false;

    if (pattern[0] == '^') {
        match_front = true;
        pattern = pattern.substr(1);
    }

    if (pattern[pattern.length()-1] == '$') {
        match_back = true;
        pattern = pattern.substr(0, pattern.length()-1);
    }
}

/** 
 * Matches string against pattern specified in constructor. By default
 * does a substring search i.e. matches anywhere in the string.
 * 
 * @param input String to match against pattern.
 * 
 * @return True if matches, false otherwise.
 */
bool SimpleMatch::matches(string input) {
    input = trim(input);

    if (input == pattern) //this takes care of both match_* being true
        return true;

    if (!match_back && !match_front && input.find(pattern) != string::npos)
        return true;

    if (match_front && !match_back && input.find(pattern) == 0)
        return true;

    if (match_back && !match_front) {
        size_t foundpos = input.rfind(pattern);
        if (foundpos != string::npos &&
            foundpos + pattern.length() == input.length())
            return true;
    }

    return false;
}

