/* $Id: Style.hh,v 1.11 2004/02/03 01:05:31 hyriand Exp $ */

/**
 * @file   Style.hh
 * @date   12 Jul 2003
 *
 * @brief Class that consolidates all the current style information.
 *
 */
#ifndef __Style_hh
#define __Style_hh

#include "Types.hh"
#include "Image.hh"
#include "Font.hh"
#include "Texture.hh"


enum {
    LeftJustify,
    RightJustify,
    CenterJustify
};

struct Padding {
    int left, right, top, bottom;

    Padding(): left(0), right(0), top(0), bottom(0) {};
};

struct ButtonStateStyle
{
    TexturePtr texture;
    KaColor    color;
    Pixmap     pixmap;
    GC         gc;

    ButtonStateStyle(): texture() {};
};

struct ButtonStyle
{
  int x, id, cb, autoplace;
  bool fg;
  std::string action;
  ButtonStateStyle focused,   focusedActive;
  ButtonStateStyle unfocused, unfocusedActive;
  ButtonStateStyle pressed,   pressedActive;
};


struct WindowStateStyle
{
  public:
    KaColor    labelColor, labelShadowColor;
    TexturePtr handleTexture, leftGripTexture, rightGripTexture, 
        titleTexture, labelTexture,
        leftEdgeTexture, rightEdgeTexture;
    KaFont     font;

    WindowStateStyle() {};
};

struct WindowStyle
{
    std::vector<ButtonStyle> buttonStyles;
    int numButtons;
    unsigned int handleThickness, borderThickness, titleHeight;
    bool roundedTopLeft, roundedTopRight, roundedBottomLeft, roundedBottomRight;    
    KaColor outlineColor, borderColor;
    WindowStateStyle focused, unfocused;
    int yPosition;
    int justify;
    int titleBarWidth, titleBarWidthPercent;
    bool fitText;
    unsigned int leftEdgeWidth, rightEdgeWidth;
    Padding buttonPadding;

    // Style uses draw: blocks which means we don't draw anything extra.
    bool usesDrawing; 
    
    WindowStyle(): roundedTopLeft(false), roundedTopRight(false),
                   roundedBottomLeft(false), roundedBottomRight(false), 
                   titleBarWidth(0), titleBarWidthPercent(0), fitText(false),
                   leftEdgeWidth(0), rightEdgeWidth(0), usesDrawing(false) {};
};

struct MenuItemStyle
{
  KaFont font, highlightFont;
  int yPosition;
};

struct MenuFrameStyle: MenuItemStyle
{
    KaColor   color, shadowColor, highlightColor, highlightShadowColor;
    TexturePtr texture, highlightTexture;
    int justify;
    
    MenuFrameStyle() {};
};

struct MenuTitleStyle
{
    KaColor     color, shadowColor;
    TexturePtr  texture;
    KaFont      font;

    int justify;
    int yPosition;

    MenuTitleStyle() {};
};

struct MenuCheckboxStyle: MenuItemStyle
{
    std::string   label;
};

struct MenuBulletStyle: MenuItemStyle
{
    std::string   label;
};

struct MenuStyle
{
  MenuFrameStyle    frame;
  MenuTitleStyle    title;
  MenuCheckboxStyle checkboxTrue;
  MenuCheckboxStyle checkboxFalse;
  MenuBulletStyle   bullet;
  bool roundedTopLeft, roundedTopRight, roundedBottomLeft, roundedBottomRight;    

  KaColor borderColor;

  unsigned int borderThickness, titleHeight, itemHeight;

  MenuStyle(): roundedTopLeft(false), roundedTopRight(false),
               roundedBottomLeft(false), roundedBottomRight(false) {};
};

struct DockholderStyle
{
  KaColor      borderColor;
  TexturePtr   texture;
  unsigned int borderThickness;
};


struct DockStyle
{
  int x;
  int y;
  int geometry;
  int direction;
  int stacking;
  long int     desktop_mask;
  unsigned int gridspace;
  std::vector<int>  order_type;
  std::vector<SimpleMatch *> order;    
  bool centered;
  bool inworkspace;
  DockholderStyle style;
};

struct Style
{
  public:
    WindowStyle       window;
    MenuStyle         menu;

    std::vector<DockStyle>   dockStyles;
};

#endif
