/* $Id: StyleLoader.hh,v 1.8 2003/11/20 22:34:28 pervago Exp $ */

/**
 * @file   StyleLoader.hh
 * @date   21 Jun 2003
 *
 * @brief Definition of StyleLoader class which handles all of the
 * style loading
 *
 */

#ifndef __StyleLoader_hh
# define __StyleLoader_hh

#include "Types.hh"
#include "DrawOp.hh"
#include "Style.hh"

extern "C" {
# include <X11/Xlib.h>
}
# include <string>


class StyleLoader
{
public:
    StyleLoader();
    virtual void loadStyle(KaScreen* kascreen)=0;
protected:
    void readDatabaseColor(const std::string&, KaColor *, unsigned long,
                           KaImageControl *);

    void readTexture(KaScreen* screen, const std::string&, DrawOps&);
    void readTexture(KaScreen* screen, const std::string&, 
                     shared_ptr<Texture>&);

    bool readDatabaseFont(const std::string&, KaFont&, KaFont&);

    ButtonStyle readButton(const std::string& prefix);

    virtual bool getValue(const std::string& name, std::string& value)=0;

    ImColor parseColor(std::string colorString, int transparency=0);

    ImColor getColor(const std::string& prefix, 
                     const std::string& suffix);

    void getIntegerValue(const std::string& name, 
                         int& value, int min, int max, int defaultValue=0);
    void getIntegerValue(const std::string& name, 
                         unsigned int& value, int min, int max, int defaultValue=0);
    void readRoundedCorners(const std::string& name, 
                            bool& topLeft, bool& topRight, 
                            bool& bottomLeft, bool& bottomRight);

    int readJustify(const std::string& name, int defaultValue=LeftJustify);

    void readFonts();

    void readPadding(const std::string& name, Padding& padding);

    KaScreen* kascreen;
    Display* display;
};


#endif // __Menu_hh
