/**
 * @file   Texture.cc
 * @date   Sun Oct  5 16:03:51 2003
 * 
 * @brief Implementation of Texture class which is a container of a
 * list of drawing operations.
 * 
 * 
 */

#include "Texture.hh"

#include <algorithm>
#include <iterator>
#include <sstream>

using namespace std;

Texture::Texture(const DrawOps& _ops) {
    copy(_ops.begin(), _ops.end(), back_inserter(ops));
}

Texture::~Texture() {
}

const shared_ptr<ImImage> Texture::render(int width, int height) {
    shared_ptr<ImImage> img(new ImImage(width, height));
    ImColor blank(0,0,0,0);

    img->set_has_alpha(true);
    img->clear(blank);

    DrawOps::const_iterator it;
    for (it = ops.begin(); it != ops.end(); ++it)
        (*it)->draw(*img);
    
    return img;
}

