/* $Id: Timer.hh,v 1.7 2003/09/11 22:42:21 faugn Exp $ */

/**
 * @file   Timer.hh
 * @date   05-Aug-2002 09:05:11
 *
 * @brief Definition of Timer and Interrupt classes
 *
 * Function declarations and variable definitions for Timer and Interrupt
 * classes.
 *
 */

#ifndef __Timer_hh
# define __Timer_hh

# include "Types.hh"

extern "C" {
# include <X11/Xlib.h>
# ifdef    TIME_WITH_SYS_TIME
#  include <sys/time.h>
#  include <time.h>
# else // !TIME_WITH_SYS_TIME
#  ifdef    HAVE_SYS_TIME_H
#   include <sys/time.h>
#  else // !HAVE_SYS_TIME_H
#   include <time.h>
#  endif // HAVE_SYS_TIME_H
# endif // TIME_WITH_SYS_TIME
}
# include <list>

class Timer {
public:
    Timer(Kahakai *);
    virtual ~Timer(void);

    void AddInterrupt(Interrupt *);
    void Start(void);
    void Pause(void);
    void ValidateInterrupts(XEvent *e);

    Kahakai *kahakai;
    std::list<Interrupt *> interrupts;
    bool paused;

private:
    struct itimerval timerval;
};

class Interrupt {
public:
    Interrupt(KaAction *, XEvent *, Window);

    Window id;
    KaMenuItem *wm;
    KaScreen *ws;
    struct timeval delay;
    KaAction *action;
    XEvent event;
};

void timeout(int);

#endif // __Timer_hh
