/* $Id: Window.hh,v 1.70 2003/12/14 18:10:35 pervago Exp $ */

/**
 * @file   Window.cc
 * @date   02-May-2001 21:43:03
 *
 * @brief Definition of KaWindow and KaChildWindow classes
 *
 * Function declarations and variable definitions for KaWindow and
 * KaChildWindow classes.
 *
 */

//TODO:  This file needs serious refactoring.
//Anytime a header file 400 lines long for a single class, there needs to be some major revisions.
//I can see several classes coming from this one.
//
// <mackstann>
// one idea i had was to make a base class called XWindow or X11Window or
// XProtoWindow or something like that, which handles all of the X-related
// stuff (methods like Close() can be done in that base class, and KaWindow
// can extend them if kahakai-specific stuff needs to be done as well). 
// the KaWindow class can then focus on the higher-level stuff.

#ifndef __Window_hh
# define __Window_hh

# include "Types.hh"

# include "Action.hh"
# include "Functions.hh"
# include "Screen.hh"
# include "WindowAttributes.hh"

# define DELETED { deleted = true; XUngrabServer(display); return; }
# define WW_DELETED { ww->deleted = true; XUngrabServer(display); return; }

# define ApplyGravity   1
# define RemoveGravity -1

typedef struct {
    int max_width;
    int max_height;
    int min_width;
    int min_height;
    int width_inc;
    int height_inc;
    int base_width;
    int base_height;
    int win_gravity;
} SizeStruct;

typedef struct {
    bool title;
    bool border;
    bool handle;
    bool sticky;
    bool shaded;
    bool max;
    bool hmax;
    bool vmax;
    bool all;
    bool alwaysontop;
    bool alwaysatbottom;
    bool focusable;
    bool tasklist;
    bool fullscreen;
    bool hidden;
} KaWindowFlags;

typedef enum {
    DesktopWindowType,
    ToolbarWindowType,
    DockWindowType,
    SplashWindowType,
    MenuWindowType,
    OtherWindowType
    // add dialog type
} NetWindowType;

class KaWindow : public WindowObject {
public:
    friend class NetHandler;

    KaWindow(Window, KaScreen *);
    virtual ~KaWindow(void);

    int GetFrameLeft();
    int GetFrameTop();
    int GetFrameBottom();
    int GetFrameRight();

    int GetFrameWidth();
    int GetFrameHeight();

    void SetFrameLeft(int x);
    void SetFrameRight(int x);
    void SetFrameTop(int y);
    void SetFrameBottom(int y);
    void SetFrameWidth(int w);
    void SetFrameHeight(int h);

    bool OnCurrentDesktop();

    void Show(void);
    void Hide(void);
    void RedrawWindow(bool = false, bool = true);
    void SendConfig(void);
    void Gravitate(int);
    void UpdateGrabs(void);
    void ButtonPressed(KaWindowButton *);
    bool IncSizeCheck(int, int, int *, int *);
    bool IncFrameSizeCheck(int, int, int *, int *);
    void IncSizeEnforce();

    //deprecated!
    inline bool IsHorizontallyVisible() {
        return (bool) ((attrib.getX() + attrib.getWidth()) > 0 &&
                       (attrib.getX() < kascreen->width));
    }
    inline bool IsVerticallyVisible() {
        return (bool) ((attrib.getY() + attrib.getHeight()) > 0 &&
                       (attrib.getY() < kascreen->height));
    }

    void DrawTitlebar(bool = false);
    void DrawHandlebar(bool = false);
    void DrawEdges();
    void FocusWin(void);
    void UnFocusWin(void);
    void Focus(bool vis = false);
    void _Maximize(int x, int y, bool horizontal=true, bool vertical=true);

    void MenuMap(const char*, bool);
    void MenuRemap(const char*, bool);
    void MenuUnmap(const char*, bool);

    void Shape(void);
    void ShapeEvent(Window, bool updateShapedFlag=true);

    void Raise();
    void Lower();
    inline void FocusVis() {
        Focus(true);
    }
    void Move(XEvent *);
    void MoveOpaque(XEvent *);
    inline void ResizeRight(XEvent *e) {
        Resize(e, EastType);
    }
    inline void ResizeLeft(XEvent *e) {
        Resize(e, WestType);
    }
    inline void ResizeRightOpaque(XEvent *e) {
        ResizeOpaque(e, EastType);
    }
    inline void ResizeLeftOpaque(XEvent *e) {
        ResizeOpaque(e, WestType);
    }

    void Resize(XEvent *, int hdir, int vdir=SouthType);
    void ResizeOpaque(XEvent *, int hdir, int vdir=SouthType);

    inline void ResizeNorthWest(XEvent *e) { Resize(e, WestType, NorthType); };
    inline void ResizeNorthEast(XEvent *e) { Resize(e, EastType, NorthType); };
    inline void ResizeSouthWest(XEvent *e) { Resize(e, WestType, SouthType); };
    inline void ResizeSouthEast(XEvent *e) { Resize(e, EastType, SouthType); };
    void SmartResize(XEvent *e, bool opaque=false);
    inline void SmartResizeOpaque(XEvent *e) { SmartResize(e, true); };

    void EndMoveResize();
    inline void Maximize() { _Maximize(-100000, -100000); } 
    inline void MaximizeHorizontally() { _Maximize(-100000, -100000, true,  false); } 
    inline void MaximizeVertically()   { _Maximize(-100000, -100000, false, true); } 
    void UnMaximize();
    void ToggleMaximize();
    void ToggleMaximizeVertically();
    void ToggleMaximizeHorizontally();
    void Close();
    void Kill();
    void CloseKill();
    inline void MenuMap(KaAction *ac) {
        MenuMap(ac->param, false);
    }
    inline void MenuMapFocused(KaAction *ac) {
        MenuMap(ac->param, true);
    }
    inline void MenuRemap(KaAction *ac) {
        MenuRemap(ac->param, false);
    }
    inline void MenuRemapFocused(KaAction *ac) {
        MenuRemap(ac->param, true);
    }
    inline void MenuUnmap(KaAction *ac) {
        MenuUnmap(ac->param, false);
    }
    inline void MenuUnmapFocus(KaAction *ac) {
        MenuUnmap(ac->param, true);
    }

    void SkipTaskList();
    void Shade(bool force = false);
    void UnShade();
    void ToggleShade();
    void Sticky();
    void UnSticky();
    void ToggleSticky();
    void Minimize(bool force = false);
    void UnMinimize();
    void ToggleMinimize();
    void ToggleMinimizeActive();
    void FullscreenOn();
    void FullscreenOff();
    void FullscreenToggle();
    void DecorTitleOn();
    void DecorHandleOn();
    void DecorBorderOn();
    void DecorAllOn();
    void DecorTitleOff();
    void DecorHandleOff();
    void DecorBorderOff();
    void DecorAllOff();
    void DecorTitleToggle();
    void DecorHandleToggle();
    void DecorBorderToggle();
    void DecorAllToggle();
    void AlwaysontopOn();
    void AlwaysatbottomOn();
    void AlwaysontopOff();
    void AlwaysatbottomOff();
    void AlwaysontopToggle();
    void AlwaysatbottomToggle();
    void AcceptConfigRequestOn();
    void AcceptConfigRequestOff();
    void AcceptConfigRequestToggle();
    inline void RaiseFocus() {
        Raise();
        Focus(true);
    }
    void MoveResize(const char* geometry);
    void MoveResizeVirtual(const char* geometry);
    void MoveWindowToPointer(XEvent *);
    void MoveWindowBy(const char* geometry);
    void MoveWindowToSmartPlace(XEvent *);
    bool SmartPlaceLeftToRight();
    void MoveWindowToViewport(const char* viewport);
    void MoveWindowToCurrentViewport();
    bool CenterOverParent();
    inline void moveToSmartPlaceIfUninitialized(XEvent *e) {
        // i don't see this function called from ANYWHERE, what's it for?

        // MoveWindowToSmartPlace() checks this anyways.
        // if (! positioned_by_app)

        MoveWindowToSmartPlace(e);
    }
    void UpdateStack(void);
    void DesktopMask(const char* mask_str);
    void JoinDesktop(const char* desktop_str);
    void PartCurrentJoinDesktop(const char *desktop_str);
    void PartDesktop(const char* desktop_str);
    void PartCurrentDesktop();
    void JoinCurrentDesktop();
    void JoinAllDesktops();
    void PartAllDesktopsExceptCurrent();
    inline void Nop() {}
    void UpdateButtons(unsigned int);
    void UpdateTitlebar(void);
    void updateDecorations();

    void EvAct(XEvent *, EventDetail *, std::vector<KaAction *> *, int);

    string labeltext; // the label, with coordinate stuff possibly added
    string name;      // the name after smart naming
    string realname;  // the REAL name

    int smartnum;

    char *host, *pid;

    int realnamelen;
    bool has_focus, want_focus, mapped, dontsend, deleted, ign_config_req,
        hidden, move_resize;
    Display *display;
    Kahakai *kahakai;
    KaScreen *kascreen;
    int border_w, title_w, handle_w, left_w, right_w, screen_number, state;
    KaChildWindow *frame, *label;
    TitleBar *title;
    std::list<KaWindowButton *> buttons;

    WindowAttributes attrib, restore_max_attrib;
    KaWindowFlags flags, restore_max_flags;

    SizeStruct size;
    NetHandler *net;
    XClassHint *classhint;
    std::list<Window> transients;

    bool render_if_transparency;

    bool IsHorizontallyOnScreen();
    bool IsVerticallyOnScreen();
    bool IsOnScreen();

    bool IsHorizontallyInWorkarea();
    bool IsVerticallyInWorkarea();
    bool IsInWorkarea();

    // formerly private
    bool IsVisible();

    unsigned desktop_mask;

private:
    Window transient_for;
    WMstrut *wm_strut;
    KaChildWindow *grip_r, *grip_l, *handle, *leftEdge, *rightEdge;
    WindowAttributes old_attrib;
    int restore_shade;
    bool positioned_by_app;
    int titleBarWidth;
    NetWindowType windowType;

    WindowFrameGeometry* geometry;

    void RestrictMaxMovement(FrameGeometry * geom = 0);

    void CheckSnap(FrameGeometry * geom = 0);

    bool CheckSnapAgainst(FrameGeometry * geom,
                          int left, int top, int right, int bottom);

    bool CheckWindowSnap(FrameGeometry * geometry = 0);
    bool CheckEdgeSnap(FrameGeometry * geometry = 0);
    bool CheckDockSnap(FrameGeometry * geometry = 0);

    bool CheckSnapAgainstEdges(FrameGeometry * geometry,
                               int left, int top, int right, int bottom);
    bool CheckResistAgainstEdges(FrameGeometry * geometry,
                               int left, int top, int right, int bottom);
    bool gone_past_bottom;
    bool gone_past_right;
    bool gone_past_left;
    bool gone_past_top;

    void ReparentWin(void);
    void InitVars(int phase);
    void GetAttributes();
    void BuildButtons(int * left_end, int * right_end);
    void BuildDecorations();
    void InitPosition(void);
    void CreateOutline(void);
    void DestroyOutline(void);
    void DrawOutline(int, int, int, int);
    void DrawFrameOutline(int, int, int, int);

    bool _MoveOpaque(XEvent *, int, int, std::list<XEvent *> *);

    void _Resize(XEvent*, int hdir, int vdir, bool opaque);

    void SetActionLists(void);
    void MapWindow(void);
    void UpdateAllAttributes(void);

    void nameUpdated();

    void restoreDecorationsFromResize();

#ifdef XINERAMA
    XineramaScreenInfo* currentXineramaScreen();
#endif

    KaImageControl *ic;
    Window o_west, o_north, o_south, o_east;
    bool sendcf;

    bool heightChanged;
    bool shadeOnUnmaximize;

    bool shaped, been_shaped;
};

#ifdef SWIG
#define SWIG_NOTABSTRACT(cls) %feature("notabstract") cls
#else
#define SWIG_NOTABSTRACT(cls)
#endif

#define EVENT_CLASS(name) SWIG_NOTABSTRACT(WindowEv##name); class WindowEv##name: public WindowFunction { public: WindowEv##name() {}; void operator()(KaWindow*, XEvent*, KaAction*); };

#define EVENT_CLASS_INLINE(name, body)  SWIG_NOTABSTRACT(WindowEv##name); class WindowEv##name: public WindowFunction  { public: WindowEv##name() {}; void operator()(KaWindow* self, XEvent* e, KaAction* ac) { body; } };

#define WRAP(name) EVENT_CLASS_INLINE(name, self->name(e, ac))
#define WRAP_E(name) EVENT_CLASS_INLINE(name, self->name(e))
#define WRAP_0(name) EVENT_CLASS_INLINE(name, self->name())
#define WRAP_A(name) EVENT_CLASS_INLINE(name, self->name(ac))
#define WRAP_APARAM(name) EVENT_CLASS_INLINE(name, self->name(ac->param))

WRAP_0(Focus);
WRAP_0(FocusVis);

EVENT_CLASS_INLINE(UnFocus, self->UnFocusWin());

WRAP_0(Close);
WRAP_0(Raise);
WRAP_0(Lower);
WRAP_APARAM(MoveResize);
WRAP_APARAM(MoveResizeVirtual);
WRAP_E(MoveWindowToPointer);
WRAP_APARAM(MoveWindowBy);
WRAP_E(MoveWindowToSmartPlace);
WRAP_APARAM(MoveWindowToViewport);
WRAP_0(MoveWindowToCurrentViewport);

WRAP_APARAM(DesktopMask);
WRAP_APARAM(JoinDesktop);
WRAP_APARAM(PartDesktop);
WRAP_APARAM(PartCurrentJoinDesktop);
WRAP_0(PartCurrentDesktop);
WRAP_0(JoinAllDesktops);
WRAP_0(PartAllDesktopsExceptCurrent);

WRAP_E(Move);
WRAP_E(MoveOpaque);
WRAP_0(RaiseFocus);
WRAP_0(Shade);
WRAP_0(UnShade);
WRAP_0(ToggleShade);

WRAP_A(MenuMap);
WRAP_A(MenuMapFocused);
WRAP_A(MenuRemap);
WRAP_A(MenuRemapFocused);
WRAP_A(MenuUnmap);
WRAP_A(MenuUnmapFocus);

WRAP_E(ResizeLeft);
WRAP_E(ResizeLeftOpaque);
WRAP_E(ResizeRight);
WRAP_E(ResizeRightOpaque);
WRAP_E(ResizeNorthWest);
WRAP_E(ResizeNorthEast);
WRAP_E(ResizeSouthWest);
WRAP_E(ResizeSouthEast);
WRAP_E(SmartResize);
WRAP_E(SmartResizeOpaque);

WRAP_0(EndMoveResize);
WRAP_0(Maximize);
WRAP_0(MaximizeHorizontally);
WRAP_0(MaximizeVertically);
WRAP_0(UnMaximize);
WRAP_0(ToggleMaximize);
WRAP_0(ToggleMaximizeVertically);
WRAP_0(ToggleMaximizeHorizontally);
WRAP_0(Kill);
WRAP_0(CloseKill);

WRAP_0(Sticky);
WRAP_0(UnSticky);
WRAP_0(ToggleSticky);

WRAP_0(Minimize);
WRAP_0(UnMinimize);
WRAP_0(ToggleMinimize);
WRAP_0(ToggleMinimizeActive);

WRAP_0(FullscreenOn);
WRAP_0(FullscreenOff);
WRAP_0(FullscreenToggle);

WRAP_0(DecorTitleOn);
WRAP_0(DecorHandleOn);
WRAP_0(DecorBorderOn);
WRAP_0(DecorAllOn);

WRAP_0(DecorTitleOff);
WRAP_0(DecorHandleOff);
WRAP_0(DecorBorderOff);
WRAP_0(DecorAllOff);

WRAP_0(DecorTitleToggle);
WRAP_0(DecorHandleToggle);
WRAP_0(DecorBorderToggle);
WRAP_0(DecorAllToggle);

WRAP_0(AlwaysontopOn);
WRAP_0(AlwaysontopOff);
WRAP_0(AlwaysontopToggle);

WRAP_0(AlwaysatbottomOn);
WRAP_0(AlwaysatbottomOff);
WRAP_0(AlwaysatbottomToggle);

WRAP_0(AcceptConfigRequestOn);
WRAP_0(AcceptConfigRequestOff);
WRAP_0(AcceptConfigRequestToggle);

#undef EVENT_CLASS
#undef EVENT_CLASS_INLINE
#undef WRAP

#endif // __Window_hh
