/* $Id: WindowAttributes.hh,v 1.11 2003/09/19 01:14:43 faugn Exp $ */

/**
 * @file   WindowAttributes.hh
 * @date   01-June-2003 17:30:35
 *
 * @brief Definition of WindowAttributes class
 *
 * Function and variable definitions for the WindowAttributes class
 *
 */

#ifndef WINDOWATTRIBUTES_H
# define WINDOWATTRIBUTES_H

# include "Types.hh"

extern "C" {
# include <X11/Xlib.h>
}

class WindowAttributes {
private:
    void init(int, int, int, int, int, int, Colormap);

public:
        //The use of these are deprecated, they are only here for backwards compatibility.  They are
        //only an aid in refactoring and will eventually become private.
        //          ***DO NOT USE THESE DIRECTLY ON THE PENALTY OF DEATH***  
    int x;
    int y;
    int width;
    int height;
    int misc0;
    int misc1;
    Colormap colormap;

    //Constructors
    WindowAttributes();
    WindowAttributes(int, int, int, int, int, int, Colormap);
    WindowAttributes(const WindowAttributes &);

    //Destructor
    ~WindowAttributes();

    // Accessors
    int getX() const;
    int getY() const;

    inline int getLeft()   const { return getX(); };
    inline int getTop()    const { return getY(); };
    inline int getRight()  const { return getLeft() + getWidth(); };
    inline int getBottom() const { return getTop() + getHeight(); };

    int getWidth() const;
    int getHeight() const;
    int getMisc0() const;
    int getMisc1() const;
    Colormap getColormap() const;

    // Mutators
    void setX(const int);
    void setY(const int);
    void setWidth(const int);
    void setHeight(const int);
    void setMisc0(const int);
    void setMisc1(const int);
    void setColormap(const Colormap);
};

#endif
