/**
 * @file   WindowIcons.cc
 * @date   Fri Dec 26 01:33:46 2003
 * 
 * @brief  Implementation of WindowIcons class which handles loading
 *         icons from _NET_WM_ICON hint, looking up (and optionally scaling)
 *         the icon of specified size.
 */

#include "WindowIcons.hh"

#include "Imlib2pp.hh"
#include "Net.hh"
#include "Window.hh"

using namespace std;

using boost::shared_ptr;

WindowIcons::WindowIcons(const KaWindow* ka) {
    // Parts of code from Openbox code (tests/icons.c)
    unsigned long offset = 0;

    while (1) {        
        long unsigned int width, height;
        Atom ret_type;
        int ret_format;
        unsigned long ret_items, ret_bytesleft;

        CARD32* ret;        

        XGetWindowProperty(ka->display, ka->id, 
                           ka->net->net_wm_icon, offset++, 1,
                           False, XA_CARDINAL, &ret_type, &ret_format,
                           &ret_items, &ret_bytesleft,
                           (unsigned char**) &ret);
        
        if (ret_type == None || ret_items < 1) {
            break;
        }

        width = ret[0];

        XFree(ret);
        
        XGetWindowProperty(ka->display, ka->id, 
                           ka->net->net_wm_icon, offset++, 1,
                           False, XA_CARDINAL, &ret_type, &ret_format,
                           &ret_items, &ret_bytesleft,
                           (unsigned char**) &ret);
        if (ret_type == None || ret_items < 1) {
            break;
        }

        height = ret[0];

        XFree(ret);

        DATA32* data = new DATA32[width * height];
        
        XGetWindowProperty(ka->display, ka->id, 
                           ka->net->net_wm_icon, offset, width * height,
                           False, XA_CARDINAL, &ret_type, &ret_format,
                           &ret_items, &ret_bytesleft,
                           (unsigned char**) &data);
        if (ret_type == None || ret_items < width * height) {
            break;
        }
        
        offset += width * height;

        shared_ptr<ImImage> img(new ImImage(width, height, data));
        img->set_has_alpha(true);

        icons.push_back(make_pair(make_pair(width, height), img));
    }

    sort(icons.begin(), icons.end());
}

bool WindowIcons::hasIcons() const {
    return !icons.empty();
}

shared_ptr<ImImage> WindowIcons::getIcon(unsigned int width, 
                                         unsigned int height) const {
    if (!hasIcons()) return shared_ptr<ImImage>();
    
    // Search from the end (biggest icons) to start (smallest icons)
    //
    // It's better to scale down than up.
    IconList::const_reverse_iterator rit = icons.rbegin();
    for (; rit != icons.rend(); ++rit) {
        // If the next biggest icon is too small or 
        // this is the last one, then stop.
        IconList::const_reverse_iterator next = rit + 1;

        if ((next == icons.rend()) ||
            (next->first.first < width || next->first.second < height)) {
            return rit->second;
        }
    }

    return icons.begin()->second;
}

