/* $Id: XrmStyleLoader.cc,v 1.16 2003/11/21 20:15:46 pervago Exp $ */

/**
 * @file   XrmStyleLoader.cc
 * @date   12 Jul 2003
 *
 * @brief Implementation of the style parser for old (waimea/box) styles.
 *
 */

#include "XrmStyleLoader.hh"
#include "Screen.hh"
#include "Resources.hh"
#include "Logger.hh"
#include "Imlib2pp.hh"

#include <iostream>
#include <utility>
#include <string>

using namespace std;

XrmStyleLoader::XrmStyleLoader()
{
}

XrmStyleLoader::~XrmStyleLoader()
{
}

/**
 * @fn    loadStyle(KaScreen *kascreen)
 * @brief Reads style file
 *
 * Reads a style resources from a style file.
 *
 * @param kascreen KaScreen to load style for
 */
void XrmStyleLoader::loadStyle(KaScreen *kascreen) {
    string value;
    int screen = kascreen->screen_number;
    WindowStyle *wstyle = &kascreen->style.window;
    MenuStyle   *mstyle = &kascreen->style.menu;
    KaImageControl *ic = kascreen->ic;
    char *__m_wastrdup_tmp;

    this->kascreen = kascreen;

    database = (XrmDatabase) NULL;

    const char* filename = kascreen->config.style_file;
    if (! (database = XrmGetFileDatabase(filename)))
        logger << warn << "can't open stylefile `" << filename
                << "' for reading" << endl;

    display  = kascreen->kahakai->display;
    
    int slen = strlen(kascreen->config.style_file) - 1;
    for (; slen >= 1 && kascreen->config.style_file[slen] != '/'; slen--);
    kascreen->config.style_file[slen] = '\0';

    readFonts();

    readTexture(kascreen, "window.title.focus", 
                wstyle->focused.titleTexture);
    readTexture(kascreen, "window.title.unfocus", 
                wstyle->unfocused.titleTexture);

    readTexture(kascreen, "window.label.focus", 
                wstyle->focused.labelTexture);
    readTexture(kascreen, "window.label.unfocus", 
                wstyle->unfocused.labelTexture);

    readTexture(kascreen, "window.handle.focus", 
                wstyle->focused.handleTexture);
    readTexture(kascreen, "window.handle.unfocus", 
                wstyle->unfocused.handleTexture);

    readTexture(kascreen, "window.grip.focus", 
                wstyle->focused.leftGripTexture);
    readTexture(kascreen, "window.grip.unfocus", 
                wstyle->unfocused.leftGripTexture);

    wstyle->focused.rightGripTexture = wstyle->focused.leftGripTexture;
    wstyle->unfocused.rightGripTexture = wstyle->unfocused.leftGripTexture;

    readDatabaseColor("window.label.focus.textColor",
                      &wstyle->focused.labelColor, BlackPixel(display, screen), ic);
    readDatabaseColor("window.label.focus.textShadowColor",
                      &wstyle->focused.labelShadowColor,
                      BlackPixel(display, screen), ic);
    readDatabaseColor("window.label.unfocus.textColor",
                      &wstyle->unfocused.labelColor, 
                      WhitePixel(display, screen),
                      ic);
    readDatabaseColor("window.label.unfocus.textShadowColor",
                      &wstyle->unfocused.labelShadowColor, 
                      BlackPixel(display, screen),
                      ic);

    getIntegerValue("window.label.focus.textShadowXOffset",
                    wstyle->focused.font.shadow_off_x, -10, 10);
    getIntegerValue("window.label.focus.textShadowYOffset",
                    wstyle->focused.font.shadow_off_y, -10, 10);

    getIntegerValue("window.label.unfocus.textShadowXOffset",
                    wstyle->unfocused.font.shadow_off_x, -10, 10);
    getIntegerValue("window.label.unfocus.textShadowYOffset",
                    wstyle->unfocused.font.shadow_off_y, -10, 10);

    wstyle->justify = LeftJustify;
    if (getValue("window.justify", value)) {
        if (strstr(value.c_str(), "right") || strstr(value.c_str(), "Right"))
            wstyle->justify = RightJustify;
        else if (strstr(value.c_str(), "center") || strstr(value.c_str(), "Center"))
            wstyle->justify = CenterJustify;
    }



    readTexture(kascreen, "menu.frame", mstyle->frame.texture);
    readTexture(kascreen, "menu.title", mstyle->title.texture);
    readTexture(kascreen, "menu.hilite", mstyle->frame.highlightTexture);

    readDatabaseColor("menu.frame.textColor",
                      &mstyle->frame.color, 
                      BlackPixel(display, screen), ic);
    readDatabaseColor("menu.frame.textShadowColor",
                      &mstyle->frame.shadowColor, 
                      BlackPixel(display, screen), ic);
    
    readDatabaseColor("menu.hilite.textColor", 
                      &mstyle->frame.highlightColor, 
                      BlackPixel(display, screen),
                      ic);
    readDatabaseColor("menu.hilite.textShadowColor",
                      &mstyle->frame.highlightShadowColor, 
                      BlackPixel(display, screen),
                      ic);
    
    readDatabaseColor("menu.title.textColor", 
                      &mstyle->title.color, 
                      BlackPixel(display, screen), ic);
    readDatabaseColor("menu.title.textShadowColor",
                      &mstyle->title.shadowColor, 
                      BlackPixel(display, screen), ic);

    getIntegerValue("menu.frame.textShadowXOffset", 
                    mstyle->frame.font.shadow_off_x, -10, 10);
    getIntegerValue("menu.frame.textShadowYOffset", 
                    mstyle->frame.font.shadow_off_y, -10, 10);

    getIntegerValue("menu.hilite.textShadowXOffset", 
                    mstyle->frame.highlightFont.shadow_off_x, -10, 10);
    getIntegerValue("menu.hilite.textShadowYOffset", 
                    mstyle->frame.highlightFont.shadow_off_y, -10, 10);

    getIntegerValue("menu.title.textShadowXOffset", 
                    mstyle->title.font.shadow_off_x, -10, 10);
    getIntegerValue("menu.title.textShadowYOffset", 
                    mstyle->title.font.shadow_off_y, -10, 10);


    mstyle->frame.justify = mstyle->title.justify = 
        readJustify("menu.justify");

    mstyle->frame.justify = readJustify("menu.frame.justify",
                                        mstyle->frame.justify);
    mstyle->title.justify = readJustify("menu.title.justify",
                                        mstyle->title.justify);

    char *look_tmp;
    unsigned int ch;
    if (getValue("menu.bullet.look", value)) {
        if (sscanf(value.c_str(), "'%u'", &ch) != 1) {
            mstyle->bullet.label = __m_wastrdup(value.c_str());
        } else {
            look_tmp = new char[2];
            sprintf(look_tmp, "%c", ch);
            mstyle->bullet.label = look_tmp;
        }
    } else {
        look_tmp = new char[2];
        sprintf(look_tmp, ">");
        mstyle->bullet.label = look_tmp;
    }

    if (getValue("menu.checkbox.true.look", value)) {
        if (sscanf(value.c_str(), "'%u'", &ch) != 1) {
            mstyle->checkboxTrue.label = __m_wastrdup(value.c_str());
        } else {
            look_tmp = new char[2];
            sprintf(look_tmp, "%c", ch);
            mstyle->checkboxTrue.label = look_tmp;
        }
    } else
        mstyle->checkboxTrue.label = __m_wastrdup("[x]");

    if (getValue("menu.checkbox.false.look", value)) {
        if (sscanf(value.c_str(), "'%u'", &ch) != 1) {
            mstyle->checkboxFalse.label = __m_wastrdup(value.c_str());
        } else {
            look_tmp = new char[2];
            sprintf(look_tmp, "%c", ch);
            mstyle->checkboxFalse.label = look_tmp;
        }
    } else
        mstyle->checkboxFalse.label = __m_wastrdup("[ ]");

    
    readDatabaseColor("borderColor",
                      &wstyle->borderColor, BlackPixel(display, screen), ic);
    mstyle->borderColor = wstyle->borderColor;

    readDatabaseColor("outlineColor",
                      &wstyle->outlineColor, WhitePixel(display, screen),
                      ic);
    mstyle->borderColor = wstyle->borderColor;
    
    if (getValue("handleWidth", value)) {
        if (sscanf(value.c_str(), "%u", &wstyle->handleThickness) != 1 ||
            wstyle->handleThickness > 6)
            wstyle->handleThickness = 6;
    } else
        wstyle->handleThickness = 6;
    
    if (getValue("borderWidth", value)) {
        if (sscanf(value.c_str(), "%u", &wstyle->borderThickness) != 1)
            wstyle->borderThickness = 1;
    } else
        wstyle->borderThickness = 1;

    mstyle->borderThickness = wstyle->borderThickness;

    if (getValue("menu.borderWidth", value))
        sscanf(value.c_str(), "%u", &mstyle->borderThickness);
    
    getIntegerValue("window.title.height", wstyle->titleHeight, 0, 50);
    getIntegerValue("menu.title.height",   mstyle->titleHeight, 0, 50);

    mstyle->itemHeight = mstyle->titleHeight;
    getIntegerValue(string("menu.item.height"), mstyle->itemHeight,  0, 50);
    
    if (getValue("rootCommand", value) && kascreen->config.rootCommand == "")
        kaexec(value.c_str(), kascreen->displaystring);

    int num = 0;
    char rc_name[50];
    vector<DockStyle>::iterator dit = kascreen->style.dockStyles.begin();
    for (; dit != kascreen->style.dockStyles.end(); ++dit, ++num) {
        dit->style.borderColor = wstyle->borderColor;
        //        dit->style.texture = wstyle->focused.titleTexture;
        dit->style.borderThickness = wstyle->borderThickness;

        sprintf(rc_name, "dockappholder.dock%d.frame.style", num);
        readTexture(kascreen, rc_name, dit->style.texture);

        sprintf(rc_name, "dockappholder.dock%d.borderWidth", num);
        if (getValue(rc_name, value)) {
            if (sscanf(value.c_str(), "%u", &dit->style.borderThickness) != 1)
                dit->style.borderThickness = wstyle->borderThickness;
        }
        sprintf(rc_name, "dockappholder.dock%d.borderColor", num);
        if (getValue(rc_name, value))
            readDatabaseColor(rc_name, &dit->style.borderColor,
                              BlackPixel(display, screen), ic);
    }

    TexturePtr tf_tmp, tu_tmp, tp_tmp;
    KaColor cf_tmp, cu_tmp, cp_tmp;

    readTexture(kascreen, "window.button.focus",
                tf_tmp);
    readTexture(kascreen, "window.button.unfocus",
                tu_tmp);
    readTexture(kascreen, "window.button.pressed",
                tp_tmp);

    readDatabaseColor("window.button.focus.picColor",
                      &cf_tmp, BlackPixel(display, screen), ic);
    readDatabaseColor("window.button.unfocus.picColor",
                      &cu_tmp, WhitePixel(display, screen), ic);
    readDatabaseColor("window.button.pressed.picColor",
                      &cp_tmp, cf_tmp.getPixel(), ic);

    vector<ButtonStyle>& buttonstyles = wstyle->buttonStyles;

    ButtonStyle b;
    
    vector<ButtonStyle>::iterator bit = buttonstyles.begin();
    for (; bit != buttonstyles.end(); ++bit) {
        bit->fg = true;
        bit->x = 0;
        bit->focused.texture = bit->focusedActive.texture = tf_tmp;
        bit->focused.color = bit->focusedActive.color = cf_tmp;
        bit->unfocused.texture = bit->unfocusedActive.texture = tu_tmp;
        bit->unfocused.color = bit->unfocusedActive.color = cu_tmp;
        bit->pressed.texture = bit->pressedActive.texture = tp_tmp;
        bit->pressed.color   = bit->pressedActive.color = cp_tmp;
    }

    map<string, pair<string, int>, ltcase> buttonTypes;
    buttonTypes["shaded"]      = make_pair("shade",    ShadeCBoxType);
    buttonTypes["maximized"]   = make_pair("maximize", MaxCBoxType);
    buttonTypes["minimized"]   = make_pair("minimize", MinCBoxType);
    buttonTypes["sticky"]      = make_pair("sticky",   StickCBoxType);
    buttonTypes["close"]       = make_pair("close",    CloseCBoxType);
    buttonTypes["fullscreen"]  = make_pair("fullscreen", FsCBoxType);
    buttonTypes["decorall"]    = make_pair("decorall", AllCBoxType);
    buttonTypes["decorborder"] = make_pair("decorborder", BorderCBoxType);
    buttonTypes["decortitle"]  = make_pair("decortitle", TitleCBoxType);

    buttonTypes["alwaysontop"]     = make_pair("alwaysontop", AOTCBoxType);
    buttonTypes["alwaysatbottom"]  = make_pair("alwaysatbottom", AABCBoxType);
    
    bool first = true, found = true;
    for (num = 0; found; ++num) {
        found = false;
        b.id = num;
        b.autoplace = EastType;
        b.cb = b.x = 0;
        b.fg = true;
        b.focused.texture = tf_tmp;
        b.focused.color = cf_tmp;
        b.unfocused.texture = tu_tmp;
        b.unfocused.color = cu_tmp;
        b.pressed.texture = tp_tmp;
        b.pressed.color = cp_tmp;

        sprintf(rc_name, "window.button%d.state", num);
        if (getValue(rc_name, value)) {
            if (first) {  first = false; }
            if (buttonTypes.count(value) > 0) {
                b.action = buttonTypes[value].first;
                b.cb     = buttonTypes[value].second;
            } else {
                b.action = value;
            }
            found = true;
        } else {
            found = false; continue;
        }
            

        sprintf(rc_name, "window.button%d.foreground", num);
        if (getValue(rc_name, value)) {
            if (first) { first = false; }
            if (! strncasecmp("true", value.c_str(), value.size())) b.fg = true;
            else b.fg = false;
            found = true;
        }
        sprintf(rc_name, "window.button%d.autoplace", num);
        if (getValue(rc_name, value)) {
            if (first) {  first = false; }
            if (! strncasecmp("Left", value.c_str(), value.size()))
                b.autoplace = WestType;
            else if(! strncasecmp("False", value.c_str(), value.size()))
                b.autoplace = 0;
            else b.autoplace = EastType;
            found = true;
        }
        sprintf(rc_name, "window.button%d.position", num);
        if (getValue(rc_name, value)) {
            if (first) {  first = false; }
            if (! sscanf(value.c_str(), "%d", &b.x)) {
                b.autoplace = EastType;
            }
            else if (b.x != 0) b.autoplace = 0;
            found = true;
        }
        sprintf(rc_name, "window.button%d.false.focus", num);
        if (getValue(rc_name, value)) {
            if (first) {  first = false; }
            readTexture(kascreen, rc_name, b.focused.texture);
            found = true;
        }        
        sprintf(rc_name, "window.button%d.false.focus.picColor", num);
        if (getValue(rc_name, value)) {
            if (first) {  first = false; }
            readDatabaseColor(rc_name, &b.focused.color,
                              BlackPixel(display, screen), ic);
            found = true;
        }
        sprintf(rc_name, "window.button%d.false.unfocus", num);
        if (getValue(rc_name, value)) {
            if (first) {  first = false; }
            readTexture(kascreen, rc_name, b.unfocused.texture);

            found = true;
        }
        sprintf(rc_name, "window.button%d.false.unfocus.picColor", num);
        if (getValue(rc_name, value)) {
            if (first) {  first = false; }
            readDatabaseColor(rc_name, &b.unfocused.color,
                              BlackPixel(display, screen), ic);
            found = true;
        }
        sprintf(rc_name, "window.button%d.false.pressed", num);
        if (getValue(rc_name, value)) {
            if (first) {  first = false; }
            readTexture(kascreen, rc_name, b.pressed.texture);
            found = true;
        }
        sprintf(rc_name, "window.button%d.false.pressed.picColor", num);
        if (getValue(rc_name, value)) {
            if (first) {  first = false; }
            readDatabaseColor(rc_name, &b.pressed.color,
                              BlackPixel(display, screen), ic);
            found = true;
        }
        b.focusedActive.texture = b.focused.texture;
        b.focusedActive.color = b.focused.color;
        b.unfocusedActive.texture = b.unfocused.texture;
        b.unfocusedActive.color = b.unfocused.color;
        b.pressedActive.texture = b.pressed.texture;
        b.pressedActive.color   = b.pressed.color;
        
        sprintf(rc_name, "window.button%d.true.focus", num);
        if (getValue(rc_name, value)) {
            if (first) {  first = false; }
            readTexture(kascreen, rc_name, b.focusedActive.texture);
            found = true;
        }        
        sprintf(rc_name, "window.button%d.true.focus.picColor", num);
        if (getValue(rc_name, value)) {
            if (first) {  first = false; }
            readDatabaseColor(rc_name, &b.focusedActive.color,
                              BlackPixel(display, screen), ic);
            found = true;
        }
        sprintf(rc_name, "window.button%d.true.unfocus", num);
        if (getValue(rc_name, value)) {
            if (first) {  first = false; }
            readTexture(kascreen, rc_name, b.unfocusedActive.texture);
            found = true;
        }
        sprintf(rc_name, "window.button%d.true.unfocus.picColor", num);
        if (getValue(rc_name, value)) {
            if (first) {  first = false; }
            readDatabaseColor(rc_name, &b.unfocusedActive.color,
                              BlackPixel(display, screen), ic);
            found = true;
        }
        sprintf(rc_name, "window.button%d.true.pressed", num);
        if (getValue(rc_name, value)) {
            if (first) {  first = false; }
            readTexture(kascreen, rc_name, b.pressedActive.texture);
            found = true;
        }
        sprintf(rc_name, "window.button%d.true.pressed.picColor", num);
        if (getValue(rc_name, value)) {
            if (first) {  first = false; }
            readDatabaseColor(rc_name, &b.pressedActive.color,
                              BlackPixel(display, screen), ic);
            found = true;
        }
        if (found) buttonstyles.push_back(b);
    }

    if (buttonstyles.empty()) {
        b.id = 0;
        b.fg = true;
        b.autoplace = WestType;
        b.action = "shade";
        b.cb = ShadeCBoxType;
        buttonstyles.push_back(b);
        
        b.id = 1;
        b.fg = true;
        b.autoplace = EastType;
        b.action = "close";
        b.cb = CloseCBoxType;
        buttonstyles.push_back(b);

        b.id = 2;
        b.fg = true;
        b.autoplace = EastType;
        b.action = "maximize";
        b.cb = MaxCBoxType;
        buttonstyles.push_back(b);
    }

    wstyle->numButtons = buttonstyles.size();

    XrmDestroyDatabase(database);
}

/**
 * @fn    capitalizeAfterDots(std::string s)
 * @brief Turns "foo.bar.blah" into "Foo.Bar.Blah"
 *
 * Capitalizes letters following periods in a string.
 *
 * @param s is the string to process
 */
std::string capitalizeAfterDots(std::string s) {
    s[0] = toupper(s[0]);

    size_t dot_index = s.find(".");

    while(dot_index != std::string::npos && dot_index < s.length() - 1) {
        s[dot_index + 1] = toupper(s[dot_index + 1]);
        dot_index = s.find(".", dot_index+1);
    }
    return s;
}


bool XrmStyleLoader::getValue(const string& rname, string& value)
{
    string rclass = capitalizeAfterDots(rname);

    char* str_rname = new char[rname.length()+1];
    strncpy(str_rname, rname.c_str(), rname.length()+1);
    
    char* str_rclass = new char[rclass.length()+1];
    strncpy(str_rclass, rclass.c_str(), rclass.length()+1);

    char* value_type; // not used
    
    XrmValue xrmValue;

    Bool result = XrmGetResource(database, str_rname, str_rclass, 
                                 &value_type, &xrmValue);
    delete[] str_rname;
    delete[] str_rclass;

    if (result) {
        value = string(xrmValue.addr);
        return true;
    } else {
        return false;
    }
}
