// $Id: swig-python.h,v 1.5 2003/06/15 16:12:04 brailsmt Exp $

// This file is not meant to be used alone, only from wrapper.i
// and wrapper.i's output

#ifdef SWIG
%feature("notabstract") PythonScreenFunction;
%feature("notabstract") PythonSimpleFunction;
#endif

class PythonFunction {
 private:
    PyObject* _pyCallable;   
 public:
    PythonFunction(PyObject* pyfunc): _pyCallable(pyfunc) {
        Py_INCREF(pyfunc);
    };
 protected:
    inline void call(PyObject* args)
    {
        PyObject* result = PyEval_CallObject(_pyCallable, args);
        Py_DECREF(args);
        
        if (result == NULL) { 
            printf("** ERROR: Python callback threw an exception:\n");
            PyErr_Print();
            return;
        }
        Py_DECREF(result);          
    }     
};

#ifndef SWIG
#define K_SWIG_WRAPPER(name, cls) SWIG_NewPointerObj((void*)name, SWIGTYPE_p_##cls, 1)
#endif

class PythonMenuFunction: public MenuFunction, public PythonFunction {
   public:  
    PythonMenuFunction(PyObject* func): PythonFunction(func) {};
    
    virtual void operator()(KaMenuItem* mi, XEvent* event, KaAction* action) {    
        PyObject* args = Py_BuildValue("(OOO)", 
                                       K_SWIG_WRAPPER(mi,     KaMenuItem), 
                                       K_SWIG_WRAPPER(event,  XEvent), 
                                       K_SWIG_WRAPPER(action, KaAction));
        call(args);
    }
};

class PythonScreenFunction: public ScreenFunction, public PythonFunction {
   public:  
    PythonScreenFunction(PyObject* func): PythonFunction(func) {};

    virtual void operator()(KaScreen* screen, XEvent* event, KaAction* action) {    
        PyObject* args = Py_BuildValue("(OOO)", 
                                       K_SWIG_WRAPPER(screen, KaScreen), 
                                       K_SWIG_WRAPPER(event,  XEvent), 
                                       K_SWIG_WRAPPER(action, KaAction));
        call(args);
    }
};

class PythonWindowFunction: public WindowFunction, public PythonFunction {
   public:  
    PythonWindowFunction(PyObject* func): PythonFunction(func) {};

   void operator()(KaWindow* window, XEvent* event, KaAction* action) {    
     PyObject* args = Py_BuildValue("(OOO)", 
                                    K_SWIG_WRAPPER(window, KaWindow), 
                                    K_SWIG_WRAPPER(event,  XEvent), 
                                    K_SWIG_WRAPPER(action, KaAction));
     call(args);
   }
};


class PythonSimpleFunction: public SimpleFunction, public PythonFunction {
   public:  
   PythonSimpleFunction(PyObject* func): PythonFunction(func) {};

   void operator()(XEvent* event, KaAction* action) {    
     PyObject* args = Py_BuildValue("(OO)", 
                                    K_SWIG_WRAPPER(event,  XEvent), 
                                    K_SWIG_WRAPPER(action, KaAction));
     call(args);
   }
};

PythonMenuFunction* createMenuFunction(PyObject* pyfunc) {
  return new PythonMenuFunction(pyfunc);
}

PythonWindowFunction* createWindowFunction(PyObject* pyfunc) {
  return new PythonWindowFunction(pyfunc);
}

PythonScreenFunction* createScreenFunction(PyObject* pyfunc) {
  return new PythonScreenFunction(pyfunc);
}

PythonSimpleFunction* createSimpleFunction(PyObject* pyfunc) {
  return new PythonSimpleFunction(pyfunc);
}
