/* $Id: swig-ruby.h,v 1.12 2004/02/04 21:53:28 brailsmt Exp $ */

// This file is not meant to be used alone, only from wrapper.i
// and wrapper.i's output

#ifndef SWIG
#define K_SWIG_WRAPPER(name, cls) SWIG_NewPointerObj((void*)name, SWIGTYPE_p_##cls, 1)
#endif

#ifdef SWIG
%feature("notabstract") RubyScreenFunction;
%feature("notabstract") RubySimpleFunction;
#endif

/// This is required to allow error handling from C++.
VALUE meth_call(VALUE arg_ary) {
	VALUE meth = rb_ary_pop(arg_ary);
	VALUE args = rb_ary_pop(arg_ary);
	printf("here");
	rb_funcall2(meth, rb_intern("call"), (int)RARRAY(args)->len, (const VALUE *)&args);

	return Qnil;
}

class RubyFunction {
	private:
		VALUE rb_meth;
	protected:
		inline void call(VALUE args) {
			int result;
			printf("Calling function");
			VALUE rb_args = rb_ary_new3(2, rb_meth, args);
			rb_protect(meth_call, rb_args, &result);

			if(result != 0) {
				printf("Error/Exception occurred in Ruby callback.\n");
			}
		}
	public:
		RubyFunction(VALUE a_method) {
			rb_meth = a_method;
			//Tell the ruby GC to not destroy the rb_meth object
			rb_gc_register_address(&rb_meth);
		}

		virtual ~RubyFunction() {
			//Now the ruby GC will collect rb_meth and destroy it
			rb_gc_unregister_address(&rb_meth);
		}
};

//Beware that RubyMenuFunction.is_a? RubyFunction or RubyMenuFunction.is_a? MenuFunction tests will
//fail in ruby, but otherwise the behavior will be correct.
class RubyMenuFunction : public RubyFunction, public MenuFunction {
	public:
		RubyMenuFunction(VALUE menu_func): RubyFunction(menu_func) {};

		virtual void operator()(KaMenuItem* mi, XEvent* event, 
				KaAction* action) {
			VALUE args = rb_ary_new3(3, 
					K_SWIG_WRAPPER(mi, KaMenuItem),
					K_SWIG_WRAPPER(event,  XEvent), 
					K_SWIG_WRAPPER(action, KaAction));
			call(args);
		}
};

class RubyScreenFunction : public RubyFunction, public ScreenFunction {
	public:
		RubyScreenFunction(VALUE screen_func): RubyFunction(screen_func) {};

		virtual void operator()(KaScreen* scr, XEvent* event, KaAction* action) {
			VALUE args = rb_ary_new3(3, 
					K_SWIG_WRAPPER(scr,    KaScreen),
					K_SWIG_WRAPPER(event,  XEvent), 
					K_SWIG_WRAPPER(action, KaAction));
			call(args);
		}
};

class RubyWindowFunction	: public RubyFunction, public WindowFunction {
	public:
		RubyWindowFunction(VALUE win_func): RubyFunction(win_func) {};

		virtual void operator()(KaWindow * win, XEvent* event, KaAction* action) {
			VALUE args = rb_ary_new3(3, 
					K_SWIG_WRAPPER(win, KaWindow),
					K_SWIG_WRAPPER(event, XEvent),
					K_SWIG_WRAPPER(action, KaAction));
			call(args);
		}
};

class RubySimpleFunction : public RubyFunction, public SimpleFunction {
	public:
		RubySimpleFunction(VALUE simple_func): RubyFunction(simple_func) {};

		virtual void operator()(XEvent* event, KaAction* action) {
			VALUE args = rb_ary_new3(2, 
					K_SWIG_WRAPPER(event,  XEvent),
					K_SWIG_WRAPPER(action, KaAction));
			call(args);
		}
};

RubyMenuFunction * createMenuFunction(VALUE rb_meth) {
	return new RubyMenuFunction(rb_meth);
}

RubyWindowFunction * createWindowFunction(VALUE rb_meth) {
	return new RubyWindowFunction(rb_meth);
}

RubyScreenFunction * createScreenFunction(VALUE rb_meth) {
	return new RubyScreenFunction(rb_meth);
}

RubySimpleFunction * createSimpleFunction(VALUE rb_meth) {
	return new RubySimpleFunction(rb_meth);
}
