/* $Id: wrapper.i,v 1.35 2004/02/02 21:47:25 brailsmt Exp $ */

%module(director="1") kahakai;
%feature("director");

%{
#ifdef HAVE_CONFIG_H
# undef PACKAGE
# undef PACKAGE_NAME
# undef PACKAGE_STRING
# undef PACKAGE_TARNAME
# undef PACKAGE_VERSION
# undef PACKAGE_BUGREPORT
# include "config.h"
#endif

#include "Font.hh"
#include "Event.hh"
#include "Window.hh"
#include "Screen.hh"
#include "Resources.hh"
#include "Functions.hh"
#include "Menu.hh"
#include "MenuItem.hh"
#include "Action.hh"
#include "Net.hh"
#include "Style.hh"

#include <X11/Xlib.h>

#include <map>
#include <set>
#include <list>
#include <string>
#include <utility>

#ifdef SWIGRUBY
#include "swig-ruby.h"
#endif

#ifdef SWIGPYTHON
#include "swig-python.h"
#endif


typedef KaScreen* KaScreen_Ptr; 
class Action;
%}

namespace std {
%include <list>
%include <map>
%include <set>
}

%include "std_string.i"
%include "std_vector.i"
//%include "std_list.i"

%include "typemaps.i"

#ifdef SWIGRUBY
%include "ruby_vector.i"

//%rename(Screen)   KaScreen;
//%rename(Menu)     KaMenu;
//%rename(XWindow)  Window;
//%rename(Window)   KaWindow;
//%rename(CPP_LIST) _CPP_LIST;
//%rename(CPP_MAP)  _CPP_MAP;
//%rename(CPP_SET)  _CPP_SET;
%rename(NET_WM_STATE_REMOVE) _NET_WM_STATE_REMOVE;
%rename(NET_WM_STATE_ADD) _NET_WM_STATE_ADD;
%rename(NET_WM_STATE_TOGGLE) _NET_WM_STATE_TOGGLE;
%rename(NET_WM_MOVERESIZE_SIZE_TOPLEFT) _NET_WM_MOVERESIZE_SIZE_TOPLEFT;
%rename(NET_WM_MOVERESIZE_SIZE_TOP) _NET_WM_MOVERESIZE_SIZE_TOP;
%rename(NET_WM_MOVERESIZE_SIZE_TOPRIGHT) _NET_WM_MOVERESIZE_SIZE_TOPRIGHT;
%rename(NET_WM_MOVERESIZE_SIZE_RIGHT) _NET_WM_MOVERESIZE_SIZE_RIGHT;
%rename(NET_WM_MOVERESIZE_SIZE_BOTTOMRIGHT) _NET_WM_MOVERESIZE_SIZE_BOTTOMRIGHT;
%rename(NET_WM_MOVERESIZE_SIZE_BOTTOM) _NET_WM_MOVERESIZE_SIZE_BOTTOM;
%rename(NET_WM_MOVERESIZE_SIZE_BOTTOMLEFT) _NET_WM_MOVERESIZE_SIZE_BOTTOMLEFT;
%rename(NET_WM_MOVERESIZE_SIZE_LEFT) _NET_WM_MOVERESIZE_SIZE_LEFT;
%rename(NET_WM_MOVERESIZE_MOVE) _NET_WM_MOVERESIZE_MOVE;
%rename(NET_WM_MOVERESIZE_SIZE_KEYBOARD) _NET_WM_MOVERESIZE_SIZE_KEYBOARD;
%rename(NET_WM_MOVERESIZE_MOVE_KEYBOARD) _NET_WM_MOVERESIZE_MOVE_KEYBOARD;

#endif

%ignore NetHandler;
%ignore StyleLoader;
%ignore NewStyleLoader;
%ignore XrmStyleLoader;
%ignore MenuLoader;
%ignore Xinerama;
%ignore ImageControl;
%ignore Timer;
%ignore DockAppHandler;
%ignore ScriptFactory;
%ignore ResourceHandler;
%ignore EventHandler;
%ignore ExpressionEvaluator;
%ignore ScreenEdge;
%ignore WMstrut;
%ignore SystrayWindow;
%ignore MReq;

%ignore KaChildWindow;
%ignore KaWindowTitle;
%ignore KaWindowHandle;
%ignore KaWindowGrip;
%ignore KaWindowButton;

class WindowObject {
public:
 WindowObject(Window win_id, int win_type);

 Window id;
 int type;
 std::vector<KaAction *> *actionlist;       
};

class Kahakai {
public:
    static Kahakai* Instance();
    KaScreen *script_current_screen;

    std::map<Window, WindowObject *> window_table;
    std::vector<KaScreen*> kascreen_list;
    unsigned long double_click, screenmask;
    char *pathenv;

    void registerWindowFunction(const char* name, WindowFunction* function);
    void registerScreenFunction(const char* name, ScreenFunction* function);
    void   registerMenuFunction(const char* name, MenuFunction* function);

#ifdef XINERAMA
    Xinerama *xinerama;   
#endif    
private:
    Kahakai();    
};

namespace std
{
    template<class T1, class T2> class pair {
    public:
       T1 first;
       T2 second;      
    };

}


%typemap(python, varin) std::string * {
    if (PyString_Check($input))
        $1 = new std::string(PyString_AsString($input));
    else
        SWIG_exception(SWIG_TypeError, "string expected");
}

%typemap(python, out) std::string * {
    $result = PyString_FromStringAndSize($1->data(),$1->size());
}

%typemap(freearg) std::string * {
  if ($1) delete $1;
}

%rename(Action)  KaAction;
%rename(command) KaAction::exec; // To avoid keyword clash
%rename(command) KaMenuItem::exec; // To avoid keyword clash

namespace std {
    template<typename K, typename T> class map {
    public:
        %rename(__len__) size;
        unsigned int size() const;
        %extend {
            T& __getitem__(const K& key) {
                return (*self)[key];
            }
            void __setitem__(const K& key, const T& x) {
                (*self)[key] = x;
            }
            void __delitem__(const K& key) {
                std::map<K,T >::iterator i = self->find(key);
                if (i != self->end())
                    self->erase(i);
                else
                    throw std::out_of_range("key not found");
            }
       }
    };          

      %template(Vector_KaScreen) vector<KaScreen*>;
      %template(Vector_KaWindow) vector<KaWindow*>;
      %template(Vector_Desktop)  vector<Desktop*>;
      %template(Vector_KaAction) vector<KaAction *>;
      %template(Vector_KaMenu)   vector<KaMenu *>;
      %template(Pair_ci)         pair<char*, int>;
      %template(Vector_pair_ci)  vector<pair<char*, int> >;
      %template(Vector_int)      vector<int>;

      %template(Vector_Vector_KaAction) vector<vector<KaAction *> >;
      %template(Map_Vector_KaAction) map<string, vector<KaAction *> >;
      %template(Map_CStr_WindowFunction) map<const char*, WindowFunction*>;
      %template(Map_CStr_ScreenFunction) map<const char*, ScreenFunction*>;
      %template(Map_CStr_MenuFunction)   map<const char*, MenuFunction*>;
}


%include <X11/X.h>

KeySym  XStringToKeysym(char* str); 
KeyCode XKeysymToKeycode(Display* display, KeySym keysym);

#ifdef SWIGRUBY
%rename(WindowManager)	Kahakai;
#endif
%include "Font.hh"
%include "Event.hh"
%include "Functions.hh"
%include "Window.hh"
%include "WindowAttributes.hh"
%include "Screen.hh"
%include "Resources.hh"
%include "Menu.hh"
%include "MenuItem.hh"
%include "Action.hh"
%include "Net.hh"
%include "Style.hh"

%feature("director") ScreenFunction;

// Grab a Python function object as a Python object.
%typemap(python,in) PyObject *pyfunc {
  if (!PyCallable_Check($input)) {
      PyErr_SetString(PyExc_TypeError, "Need a callable object!");
      return NULL;
  }
  $1 = $input;
}

struct timeval
  {
    long tv_sec;		/* Seconds.  */
    long tv_usec;	/* Microseconds.  */
  };

typedef struct {
    char *res_name;
    char *res_class;
} XClassHint;

typedef struct _XDisplay Display;

static std::vector<std::pair<char*, int> > getXModifiers(Display* display);

%{
typedef struct _XDisplay Display;

static std::vector<std::pair<char*, int> > getXModifiers(Display* display) {
    const XModifierKeymap* const modmap = XGetModifierMapping(display);
    std::vector<std::pair<char*, int> > mods;
 
    if (modmap && modmap->max_keypermod > 0) {
        const int mask_table[] = { 
            ShiftMask, LockMask, ControlMask, Mod1Mask,
            Mod2Mask, Mod3Mask, Mod4Mask, Mod5Mask
        };    
        const size_t size = (sizeof(mask_table) / sizeof(mask_table[0])) *
            modmap->max_keypermod;
        
        for (size_t i = 0; i < size; ++i) {
            if (! modmap->modifiermap[i]) continue;
            KeySym ksym = XKeycodeToKeysym(display, modmap->modifiermap[i], 0);
            if (ksym) {
                char *kstring = XKeysymToString(ksym);
                if (kstring) {
                    int modmask = mask_table[i / modmap->max_keypermod];
                    mods.push_back(std::make_pair(kstring, modmask));
                }
            }
        }
        if (modmap) XFreeModifiermap(const_cast<XModifierKeymap*>(modmap));
    }
    return mods;
}
%}

#ifdef SWIGPYTHON
%include "swig-python.h" 
#endif

#ifdef SWIGRUBY
%include "swig-ruby.h"
#endif
